@@ -15,6 +15,8 @@ import enum
 import typing as ty
 import warnings
 
+from keystoneauth1 import adapter
+
 from openstack.baremetal.v1 import _common
 from openstack import exceptions
 from openstack import resource
@@ -452,7 +454,7 @@ class Node(_common.Resource):
 
         version = None
         if target in _common.PROVISIONING_VERSIONS:
-            version = '1.%d' % _common.PROVISIONING_VERSIONS[target]
+            version = f'1.{_common.PROVISIONING_VERSIONS[target]}'
 
         if config_drive:
             # Some config drive actions require a higher version.
@@ -527,16 +529,14 @@ class Node(_common.Resource):
         if service_steps is not None:
             if target != 'service':
                 raise ValueError(
-                    'Service steps can only be provided with '
-                    '"service" target'
+                    'Service steps can only be provided with "service" target'
                 )
             body['service_steps'] = service_steps
 
         if rescue_password is not None:
             if target != 'rescue':
                 raise ValueError(
-                    'Rescue password can only be provided with '
-                    '"rescue" target'
+                    'Rescue password can only be provided with "rescue" target'
                 )
             body['rescue_password'] = rescue_password
 
@@ -817,7 +817,7 @@ class Node(_common.Resource):
 
     def attach_vif(
         self,
-        session,
+        session: adapter.Adapter,
         vif_id: str,
         retry_on_conflict: bool = True,
         *,
