@@ -10,6 +10,7 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+
 from openstack.common import metadata
 from openstack import exceptions
 from openstack import format
@@ -25,6 +26,7 @@ class Volume(resource.Resource, metadata.MetadataMixin):
     _query_mapping = resource.QueryParameters(
         'name',
         'status',
+        'user_id',
         'project_id',
         'created_at',
         'updated_at',
@@ -111,15 +113,15 @@ class Volume(resource.Resource, metadata.MetadataMixin):
 
     _max_microversion = "3.60"
 
-    def _action(self, session, body, microversion=None):
+    def _action(self, session, body, microversion=None, action='patch'):
         """Preform volume actions given the message body."""
         # NOTE: This is using Volume.base_path instead of self.base_path
         # as both Volume and VolumeDetail instances can be acted on, but
         # the URL used is sans any additional /detail/ part.
         url = utils.urljoin(Volume.base_path, self.id, 'action')
-        resp = session.post(
-            url, json=body, microversion=self._max_microversion
-        )
+        if microversion is None:
+            microversion = self._get_microversion(session, action=action)
+        resp = session.post(url, json=body, microversion=microversion)
         exceptions.raise_from_response(resp)
         return resp
 
@@ -138,15 +140,33 @@ class Volume(resource.Resource, metadata.MetadataMixin):
         body = {'os-update_readonly_flag': {'readonly': readonly}}
         self._action(session, body)
 
-    def reset_status(self, session, status, attach_status, migration_status):
+    def set_image_metadata(self, session, metadata):
+        """Sets image metadata key-value pairs on the volume"""
+        body = {'os-set_image_metadata': metadata}
+        self._action(session, body)
+
+    def delete_image_metadata(self, session):
+        """Remove all image metadata from the volume"""
+        for key in self.metadata:
+            body = {'os-unset_image_metadata': key}
+            self._action(session, body)
+
+    def delete_image_metadata_item(self, session, key):
+        """Remove a single image metadata from the volume"""
+        body = {'os-unset_image_metadata': key}
+        self._action(session, body)
+
+    def reset_status(
+        self, session, status=None, attach_status=None, migration_status=None
+    ):
         """Reset volume statuses (admin operation)"""
-        body = {
-            'os-reset_status': {
-                'status': status,
-                'attach_status': attach_status,
-                'migration_status': migration_status,
-            }
-        }
+        body: dict[str, dict[str, str]] = {'os-reset_status': {}}
+        if status:
+            body['os-reset_status']['status'] = status
+        if attach_status:
+            body['os-reset_status']['attach_status'] = attach_status
+        if migration_status:
+            body['os-reset_status']['migration_status'] = migration_status
         self._action(session, body)
 
     def revert_to_snapshot(self, session, snapshot_id):
@@ -339,7 +359,9 @@ class Volume(resource.Resource, metadata.MetadataMixin):
 
         self._action(session, body)
 
-    def _prepare_request_body(self, patch, prepend_key):
+    def _prepare_request_body(
+        self, patch, prepend_key, *, resource_request_key=None
+    ):
         body = self._body.dirty
         # Scheduler hints is external to the standard volume request
         # so pass it separately and not under the volume JSON object.
