@@ -37,8 +37,11 @@ from openstack.config import vendors
 from openstack import exceptions
 from openstack import warnings as os_warnings
 
+if ty.TYPE_CHECKING:
+    from keystoneauth1.loading._plugins.identity import v3 as v3_loaders
+
 PLATFORMDIRS = platformdirs.PlatformDirs(
-    'openstack', 'OpenStack', multipath='/etc'
+    'openstack', 'OpenStack', multipath=True
 )
 CONFIG_HOME = PLATFORMDIRS.user_config_dir
 CACHE_PATH = PLATFORMDIRS.user_cache_dir
@@ -132,7 +135,7 @@ def _fix_argv(argv):
                 argv[index] = "=".join(split_args)
             # Save both for later so we can throw an error about dupes
             processed[new].add(orig)
-    overlap: ty.List[str] = []
+    overlap: list[str] = []
     for new, old in processed.items():
         if len(old) > 1:
             overlap.extend(old)
@@ -303,8 +306,8 @@ class OpenStackConfig:
         self._cache_expiration_time = 0
         self._cache_path = CACHE_PATH
         self._cache_class = 'dogpile.cache.null'
-        self._cache_arguments: ty.Dict[str, ty.Any] = {}
-        self._cache_expirations: ty.Dict[str, int] = {}
+        self._cache_arguments: dict[str, ty.Any] = {}
+        self._cache_expirations: dict[str, int] = {}
         self._influxdb_config = {}
         if 'cache' in self.cloud_config:
             cache_settings = _util.normalize_keys(self.cloud_config['cache'])
@@ -537,7 +540,7 @@ class OpenStackConfig:
             return self._expand_regions(regions)
         else:
             # crappit. we don't have a region defined.
-            new_cloud: ty.Dict[str, ty.Any] = {}
+            new_cloud: dict[str, ty.Any] = {}
             our_cloud = self.cloud_config['clouds'].get(cloud, {})
             self._expand_vendor_profile(cloud, new_cloud, our_cloud)
             if 'regions' in new_cloud and new_cloud['regions']:
@@ -1061,7 +1064,9 @@ class OpenStackConfig:
             # doing what they want causes them sorrow.
             config['auth_type'] = 'admin_token'
 
-        loader = loading.get_plugin_loader(config['auth_type'])
+        loader: loading.BaseLoader[ty.Any] = loading.get_plugin_loader(
+            config['auth_type']
+        )
 
         # As the name would suggest, v3multifactor uses multiple factors for
         # authentication. As a result, we need to register the configuration
@@ -1074,6 +1079,9 @@ class OpenStackConfig:
         # options in keystoneauth1.loading._plugins.identity.v3.MultiAuth
         # without calling 'load_from_options'.
         if config['auth_type'] == 'v3multifactor':
+            if ty.TYPE_CHECKING:
+                # narrow types
+                assert isinstance(loader, v3_loaders.MultiFactor)
             # We use '.get' since we can't be sure this key is set yet -
             # validation happens later, in _validate_auth
             loader._methods = config.get('auth_methods')
