@@ -10,6 +10,8 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+import typing as ty
+
 from openstack.container_infrastructure_management.v1 import (
     cluster as _cluster,
 )
@@ -23,6 +25,7 @@ from openstack.container_infrastructure_management.v1 import (
     service as _service,
 )
 from openstack import proxy
+from openstack import resource
 
 
 class Proxy(proxy.Proxy):
@@ -32,6 +35,8 @@ class Proxy(proxy.Proxy):
         "service": _service.Service,
     }
 
+    # ========== Clusters ==========
+
     def create_cluster(self, **attrs):
         """Create a new cluster from attributes
 
@@ -51,9 +56,9 @@ class Proxy(proxy.Proxy):
             :class:`~openstack.container_infrastructure_management.v1.cluster.Cluster`
             instance.
         :param bool ignore_missing: When set to ``False``
-            :class:`~openstack.exceptions.ResourceNotFound` will be raised when
-            the cluster does not exist. When set to ``True``, no exception will
-            be set when attempting to delete a nonexistent cluster.
+            :class:`~openstack.exceptions.NotFoundException` will be raised
+            when the cluster does not exist. When set to ``True``, no exception
+            will be set when attempting to delete a nonexistent cluster.
         :returns: ``None``
         """
         self._delete(_cluster.Cluster, cluster, ignore_missing=ignore_missing)
@@ -63,7 +68,7 @@ class Proxy(proxy.Proxy):
 
         :param name_or_id: The name or ID of a cluster.
         :param bool ignore_missing: When set to ``False``
-            :class:`~openstack.exceptions.ResourceNotFound` will be
+            :class:`~openstack.exceptions.NotFoundException` will be
             raised when the resource does not exist.
             When set to ``True``, None will be returned when
             attempting to find a nonexistent resource.
@@ -86,7 +91,7 @@ class Proxy(proxy.Proxy):
 
         :returns: One
             :class:`~openstack.container_infrastructure_management.v1.cluster.Cluster`
-        :raises: :class:`~openstack.exceptions.ResourceNotFound`
+        :raises: :class:`~openstack.exceptions.NotFoundException`
             when no resource can be found.
         """
         return self._get(_cluster.Cluster, cluster)
@@ -119,6 +124,7 @@ class Proxy(proxy.Proxy):
         return self._update(_cluster.Cluster, cluster, **attrs)
 
     # ============== Cluster Templates ==============
+
     def create_cluster_template(self, **attrs):
         """Create a new cluster_template from attributes
 
@@ -139,8 +145,8 @@ class Proxy(proxy.Proxy):
             :class:`~openstack.container_infrastructure_management.v1.cluster_template.ClusterTemplate`
             instance.
         :param bool ignore_missing: When set to ``False``
-            :class:`~openstack.exceptions.ResourceNotFound` will be raised when
-            the cluster_template does not exist. When set to ``True``, no
+            :class:`~openstack.exceptions.NotFoundException` will be raised
+            when the cluster_template does not exist. When set to ``True``, no
             exception will be set when attempting to delete a nonexistent
             cluster_template.
         :returns: ``None``
@@ -156,7 +162,7 @@ class Proxy(proxy.Proxy):
 
         :param name_or_id: The name or ID of a cluster_template.
         :param bool ignore_missing: When set to ``False``
-            :class:`~openstack.exceptions.ResourceNotFound` will be
+            :class:`~openstack.exceptions.NotFoundException` will be
             raised when the resource does not exist.
             When set to ``True``, None will be returned when
             attempting to find a nonexistent resource.
@@ -180,7 +186,7 @@ class Proxy(proxy.Proxy):
 
         :returns: One
             :class:`~openstack.container_infrastructure_management.v1.cluster_template.ClusterTemplate`
-        :raises: :class:`~openstack.exceptions.ResourceNotFound`
+        :raises: :class:`~openstack.exceptions.NotFoundException`
             when no resource can be found.
         """
         return self._get(_cluster_template.ClusterTemplate, cluster_template)
@@ -215,6 +221,7 @@ class Proxy(proxy.Proxy):
         )
 
     # ============== Cluster Certificates ==============
+
     def create_cluster_certificate(self, **attrs):
         """Create a new cluster_certificate from CSR
 
@@ -237,12 +244,13 @@ class Proxy(proxy.Proxy):
 
         :returns: One
             :class:`~openstack.container_infrastructure_management.v1.cluster_certificate.ClusterCertificate`
-        :raises: :class:`~openstack.exceptions.ResourceNotFound`
+        :raises: :class:`~openstack.exceptions.NotFoundException`
             when no resource can be found.
         """
         return self._get(_cluster_cert.ClusterCertificate, cluster_certificate)
 
     # ============== Services ==============
+
     def services(self):
         """Return a generator of services
 
@@ -251,3 +259,66 @@ class Proxy(proxy.Proxy):
             :class:`~openstack.container_infrastructure_management.v1.service.Service`
         """
         return self._list(_service.Service)
+
+    # ========== Utilities ==========
+
+    def wait_for_status(
+        self,
+        res: resource.ResourceT,
+        status: str,
+        failures: ty.Optional[list[str]] = None,
+        interval: ty.Union[int, float, None] = 2,
+        wait: ty.Optional[int] = None,
+        attribute: str = 'status',
+        callback: ty.Optional[ty.Callable[[int], None]] = None,
+    ) -> resource.ResourceT:
+        """Wait for the resource to be in a particular status.
+
+        :param session: The session to use for making this request.
+        :param resource: The resource to wait on to reach the status. The
+            resource must have a status attribute specified via ``attribute``.
+        :param status: Desired status of the resource.
+        :param failures: Statuses that would indicate the transition
+            failed such as 'ERROR'. Defaults to ['ERROR'].
+        :param interval: Number of seconds to wait between checks.
+        :param wait: Maximum number of seconds to wait for transition.
+            Set to ``None`` to wait forever.
+        :param attribute: Name of the resource attribute that contains the
+            status.
+        :param callback: A callback function. This will be called with a single
+            value, progress. This is API specific but is generally a percentage
+            value from 0-100.
+
+        :return: The updated resource.
+        :raises: :class:`~openstack.exceptions.ResourceTimeout` if the
+            transition to status failed to occur in ``wait`` seconds.
+        :raises: :class:`~openstack.exceptions.ResourceFailure` if the resource
+            transitioned to one of the states in ``failures``.
+        :raises: :class:`~AttributeError` if the resource does not have a
+            ``status`` attribute
+        """
+        return resource.wait_for_status(
+            self, res, status, failures, interval, wait, attribute, callback
+        )
+
+    def wait_for_delete(
+        self,
+        res: resource.ResourceT,
+        interval: int = 2,
+        wait: int = 120,
+        callback: ty.Optional[ty.Callable[[int], None]] = None,
+    ) -> resource.ResourceT:
+        """Wait for a resource to be deleted.
+
+        :param res: The resource to wait on to be deleted.
+        :param interval: Number of seconds to wait before to consecutive
+            checks.
+        :param wait: Maximum number of seconds to wait before the change.
+        :param callback: A callback function. This will be called with a single
+            value, progress, which is a percentage value from 0-100.
+
+        :returns: The resource is returned on success.
+        :raises: :class:`~openstack.exceptions.ResourceTimeout` if transition
+            to delete failed to occur in the specified seconds.
+        """
+        return resource.wait_for_delete(self, res, interval, wait, callback)
