@@ -20,62 +20,66 @@ import json
 import re
 import typing as ty
 
+import requests
 from requests import exceptions as _rex
 
+if ty.TYPE_CHECKING:
+    from openstack import resource
+
 
 class SDKException(Exception):
     """The base exception class for all exceptions this library raises."""
 
-    def __init__(self, message=None, extra_data=None):
+    def __init__(
+        self, message: ty.Optional[str] = None, extra_data: ty.Any = None
+    ):
         self.message = self.__class__.__name__ if message is None else message
         self.extra_data = extra_data
-        super(SDKException, self).__init__(self.message)
+        super().__init__(self.message)
 
 
 class EndpointNotFound(SDKException):
     """A mismatch occurred between what the client and server expect."""
 
-    def __init__(self, message=None):
-        super(EndpointNotFound, self).__init__(message)
+    def __init__(self, message: ty.Optional[str] = None):
+        super().__init__(message)
 
 
 class InvalidResponse(SDKException):
     """The response from the server is not valid for this request."""
 
-    def __init__(self, response):
-        super(InvalidResponse, self).__init__()
-        self.response = response
+    def __init__(self, message: ty.Optional[str] = None):
+        super().__init__(message)
 
 
 class InvalidRequest(SDKException):
     """The request to the server is not valid."""
 
-    def __init__(self, message=None):
-        super(InvalidRequest, self).__init__(message)
+    def __init__(self, message: ty.Optional[str] = None):
+        super().__init__(message)
 
 
 class HttpException(SDKException, _rex.HTTPError):
     """The base exception for all HTTP error responses."""
 
+    source: str
+    status_code: ty.Optional[int]
+
     def __init__(
         self,
-        message='Error',
-        response=None,
-        http_status=None,
-        details=None,
-        request_id=None,
+        message: ty.Optional[str] = 'Error',
+        response: ty.Optional[requests.Response] = None,
+        http_status: ty.Optional[int] = None,
+        details: ty.Optional[str] = None,
+        request_id: ty.Optional[str] = None,
     ):
         # TODO(shade) Remove http_status parameter and the ability for response
         # to be None once we're not mocking Session everywhere.
         if not message:
             if response is not None:
-                message = "{name}: {code}".format(
-                    name=self.__class__.__name__, code=response.status_code
-                )
+                message = f"{self.__class__.__name__}: {response.status_code}"
             else:
-                message = "{name}: Unknown error".format(
-                    name=self.__class__.__name__
-                )
+                message = f"{self.__class__.__name__}: Unknown error"
 
         # Call directly rather than via super to control parameters
         SDKException.__init__(self, message=message)
@@ -94,7 +98,7 @@ class HttpException(SDKException, _rex.HTTPError):
         if self.status_code is not None and (400 <= self.status_code < 500):
             self.source = "Client"
 
-    def __str__(self):
+    def __str__(self) -> str:
         # 'Error' is the default value for self.message. If self.message isn't
         # 'Error', then someone has set a more informative error message
         # and we should use it. If it is 'Error', then we should construct a
@@ -102,24 +106,23 @@ class HttpException(SDKException, _rex.HTTPError):
         if not self.url or self.message == 'Error':
             return self.message
         if self.url:
-            remote_error = "{source} Error for url: {url}".format(
-                source=self.source, url=self.url
-            )
+            remote_error = f"{self.source} Error for url: {self.url}"
             if self.details:
                 remote_error += ', '
         if self.details:
             remote_error += str(self.details)
 
-        return "{message}: {remote_error}".format(
-            message=super(HttpException, self).__str__(),
-            remote_error=remote_error,
-        )
+        return f"{super().__str__()}: {remote_error}"
 
 
 class BadRequestException(HttpException):
     """HTTP 400 Bad Request."""
 
 
+class NotFoundException(HttpException):
+    """HTTP 404 Not Found."""
+
+
 class ForbiddenException(HttpException):
     """HTTP 403 Forbidden Request."""
 
@@ -135,32 +138,25 @@ class PreconditionFailedException(HttpException):
 class MethodNotSupported(SDKException):
     """The resource does not support this operation type."""
 
-    def __init__(self, resource, method):
+    def __init__(
+        self,
+        resource: ty.Union['resource.Resource', type['resource.Resource']],
+        method: str,
+    ):
         # This needs to work with both classes and instances.
         try:
             name = resource.__name__
         except AttributeError:
             name = resource.__class__.__name__
 
-        message = 'The %s method is not supported for %s.%s' % (
-            method,
-            resource.__module__,
-            name,
-        )
-        super(MethodNotSupported, self).__init__(message=message)
+        message = f'The {method} method is not supported for {resource.__module__}.{name}'
+        super().__init__(message=message)
 
 
 class DuplicateResource(SDKException):
     """More than one resource exists with that name."""
 
 
-class ResourceNotFound(HttpException):
-    """No resource exists with that name or ID."""
-
-
-NotFoundException = ResourceNotFound
-
-
 class ResourceTimeout(SDKException):
     """Timeout waiting for resource."""
 
@@ -173,14 +169,14 @@ class InvalidResourceQuery(SDKException):
     """Invalid query params for resource."""
 
 
-def _extract_message(obj):
+def _extract_message(obj: ty.Any) -> ty.Optional[str]:
     if isinstance(obj, dict):
         # Most of services: compute, network
         if obj.get('message'):
-            return obj['message']
+            return str(obj['message'])
         # Ironic starting with Stein
         elif obj.get('faultstring'):
-            return obj['faultstring']
+            return str(obj['faultstring'])
     elif isinstance(obj, str):
         # Ironic before Stein has double JSON encoding, nobody remembers why.
         try:
@@ -189,14 +185,18 @@ def _extract_message(obj):
             pass
         else:
             return _extract_message(obj)
+    return None
 
 
-def raise_from_response(response, error_message=None):
+def raise_from_response(
+    response: requests.Response,
+    error_message: ty.Optional[str] = None,
+) -> None:
     """Raise an instance of an HTTPException based on keystoneauth response."""
     if response.status_code < 400:
         return
 
-    cls: ty.Type[SDKException]
+    cls: type[SDKException]
     if response.status_code == 400:
         cls = BadRequestException
     elif response.status_code == 403:
@@ -236,7 +236,7 @@ def raise_from_response(response, error_message=None):
                 messages.append(message)
 
         # Return joined string separated by colons.
-        details = ': '.join(messages)
+        details = ': '.join(msg for msg in messages if msg)
 
     if not details:
         details = response.reason if response.reason else response.text
@@ -275,3 +275,4 @@ class ServiceDiscoveryException(SDKException):
 
 # Backwards compatibility
 OpenStackCloudException = SDKException
+ResourceNotFound = NotFoundException
