@@ -11,11 +11,13 @@
 # under the License.
 
 import os
+import typing as ty
 import warnings
 
 from openstack import exceptions as exc
 from openstack.image.v1 import image as _image
 from openstack import proxy
+from openstack import resource
 from openstack import utils
 from openstack import warnings as os_warnings
 
@@ -258,7 +260,7 @@ class Proxy(proxy.Proxy):
         """
         warnings.warn(
             "upload_image is deprecated. Use create_image instead.",
-            os_warnings.OpenStackDeprecationWarning,
+            os_warnings.RemovedInSDK50Warning,
         )
         return self._create(_image.Image, **attrs)
 
@@ -295,7 +297,7 @@ class Proxy(proxy.Proxy):
             image = self._connection._get_and_munchify(
                 'image',
                 self.put(
-                    '/images/{id}'.format(id=image.id),
+                    f'/images/{image.id}',
                     headers=headers,
                     data=image_data,
                 ),
@@ -303,7 +305,7 @@ class Proxy(proxy.Proxy):
         except exc.HttpException:
             self.log.debug("Deleting failed upload of image %s", name)
             try:
-                self.delete('/images/{id}'.format(id=image.id))
+                self.delete(f'/images/{image.id}')
             except exc.HttpException:
                 # We're just trying to clean up - if it doesn't work - shrug
                 self.log.warning(
@@ -322,7 +324,7 @@ class Proxy(proxy.Proxy):
         :param image: The value can be either the ID of an image or a
             :class:`~openstack.image.v1.image.Image` instance.
         :param bool ignore_missing: When set to ``False``
-            :class:`~openstack.exceptions.ResourceNotFound` will be
+            :class:`~openstack.exceptions.NotFoundException` will be
             raised when the image does not exist.
             When set to ``True``, no exception will be set when
             attempting to delete a nonexistent image.
@@ -336,7 +338,7 @@ class Proxy(proxy.Proxy):
 
         :param name_or_id: The name or ID of a image.
         :param bool ignore_missing: When set to ``False``
-            :class:`~openstack.exceptions.ResourceNotFound` will be
+            :class:`~openstack.exceptions.NotFoundException` will be
             raised when the resource does not exist.
             When set to ``True``, None will be returned when
             attempting to find a nonexistent resource.
@@ -353,7 +355,7 @@ class Proxy(proxy.Proxy):
             :class:`~openstack.image.v1.image.Image` instance.
 
         :returns: One :class:`~openstack.image.v1.image.Image`
-        :raises: :class:`~openstack.exceptions.ResourceNotFound`
+        :raises: :class:`~openstack.exceptions.NotFoundException`
             when no resource can be found.
         """
         return self._get(_image.Image, image)
@@ -434,10 +436,10 @@ class Proxy(proxy.Proxy):
         img_props = {}
         for k, v in iter(properties.items()):
             if image.properties.get(k, None) != v:
-                img_props['x-image-meta-{key}'.format(key=k)] = v
+                img_props[f'x-image-meta-{k}'] = v
         if not img_props:
             return False
-        self.put('/images/{id}'.format(id=image.id), headers=img_props)
+        self.put(f'/images/{image.id}', headers=img_props)
         return True
 
     def update_image_properties(
@@ -469,7 +471,70 @@ class Proxy(proxy.Proxy):
         for k, v in iter(kwargs.items()):
             if v and k in ['ramdisk', 'kernel']:
                 v = self._connection.get_image_id(v)
-                k = '{0}_id'.format(k)
+                k = f'{k}_id'
             img_props[k] = v
 
         return self._update_image_properties(image, meta, img_props)
+
+    # ========== Utilities ==========
+
+    def wait_for_status(
+        self,
+        res: resource.ResourceT,
+        status: str,
+        failures: ty.Optional[list[str]] = None,
+        interval: ty.Union[int, float, None] = 2,
+        wait: ty.Optional[int] = None,
+        attribute: str = 'status',
+        callback: ty.Optional[ty.Callable[[int], None]] = None,
+    ) -> resource.ResourceT:
+        """Wait for the resource to be in a particular status.
+
+        :param session: The session to use for making this request.
+        :param resource: The resource to wait on to reach the status. The
+            resource must have a status attribute specified via ``attribute``.
+        :param status: Desired status of the resource.
+        :param failures: Statuses that would indicate the transition
+            failed such as 'ERROR'. Defaults to ['ERROR'].
+        :param interval: Number of seconds to wait between checks.
+        :param wait: Maximum number of seconds to wait for transition.
+            Set to ``None`` to wait forever.
+        :param attribute: Name of the resource attribute that contains the
+            status.
+        :param callback: A callback function. This will be called with a single
+            value, progress. This is API specific but is generally a percentage
+            value from 0-100.
+
+        :return: The updated resource.
+        :raises: :class:`~openstack.exceptions.ResourceTimeout` if the
+            transition to status failed to occur in ``wait`` seconds.
+        :raises: :class:`~openstack.exceptions.ResourceFailure` if the resource
+            transitioned to one of the states in ``failures``.
+        :raises: :class:`~AttributeError` if the resource does not have a
+            ``status`` attribute
+        """
+        return resource.wait_for_status(
+            self, res, status, failures, interval, wait, attribute, callback
+        )
+
+    def wait_for_delete(
+        self,
+        res: resource.ResourceT,
+        interval: int = 2,
+        wait: int = 120,
+        callback: ty.Optional[ty.Callable[[int], None]] = None,
+    ) -> resource.ResourceT:
+        """Wait for a resource to be deleted.
+
+        :param res: The resource to wait on to be deleted.
+        :param interval: Number of seconds to wait before to consecutive
+            checks.
+        :param wait: Maximum number of seconds to wait before the change.
+        :param callback: A callback function. This will be called with a single
+            value, progress, which is a percentage value from 0-100.
+
+        :returns: The resource is returned on success.
+        :raises: :class:`~openstack.exceptions.ResourceTimeout` if transition
+            to delete failed to occur in the specified seconds.
+        """
+        return resource.wait_for_delete(self, res, interval, wait, callback)
