@@ -10,6 +10,8 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+import typing as ty
+
 from openstack.message.v2 import claim as _claim
 from openstack.message.v2 import message as _message
 from openstack.message.v2 import queue as _queue
@@ -45,7 +47,7 @@ class Proxy(proxy.Proxy):
             :class:`~openstack.message.v2.queue.Queue` instance.
 
         :returns: One :class:`~openstack.message.v2.queue.Queue`
-        :raises: :class:`~openstack.exceptions.ResourceNotFound` when no
+        :raises: :class:`~openstack.exceptions.NotFoundException` when no
             queue matching the name could be found.
         """
         return self._get(_queue.Queue, queue)
@@ -73,7 +75,7 @@ class Proxy(proxy.Proxy):
         :param value: The value can be either the name of a queue or a
             :class:`~openstack.message.v2.queue.Queue` instance.
         :param bool ignore_missing: When set to ``False``
-            :class:`~openstack.exceptions.ResourceNotFound` will be
+            :class:`~openstack.exceptions.NotFoundException` will be
             raised when the queue does not exist.
             When set to ``True``, no exception will be set when
             attempting to delete a nonexistent queue.
@@ -127,7 +129,7 @@ class Proxy(proxy.Proxy):
             :class:`~openstack.message.v2.message.Message` instance.
 
         :returns: One :class:`~openstack.message.v2.message.Message`
-        :raises: :class:`~openstack.exceptions.ResourceNotFound` when no
+        :raises: :class:`~openstack.exceptions.NotFoundException` when no
             message matching the criteria could be found.
         """
         message = self._get_resource(
@@ -148,7 +150,7 @@ class Proxy(proxy.Proxy):
             the claim seizing the message. If None, the message has
             not been claimed.
         :param bool ignore_missing: When set to ``False``
-            :class:`~openstack.exceptions.ResourceNotFound` will be
+            :class:`~openstack.exceptions.NotFoundException` will be
             raised when the message does not exist.
             When set to ``True``, no exception will be set when
             attempting to delete a nonexistent message.
@@ -206,7 +208,7 @@ class Proxy(proxy.Proxy):
             :class:`~openstack.message.v2.subscription.Subscription` instance.
 
         :returns: One :class:`~openstack.message.v2.subscription.Subscription`
-        :raises: :class:`~openstack.exceptions.ResourceNotFound` when no
+        :raises: :class:`~openstack.exceptions.NotFoundException` when no
             subscription matching the criteria could be found.
         """
         subscription = self._get_resource(
@@ -223,7 +225,7 @@ class Proxy(proxy.Proxy):
             :class:`~openstack.message.v2.subscription.Subscription`
             instance.
         :param bool ignore_missing: When set to ``False``
-            :class:`~openstack.exceptions.ResourceNotFound` will be
+            :class:`~openstack.exceptions.NotFoundException` will be
             raised when the subscription does not exist.
             When set to ``True``, no exception will be thrown when
             attempting to delete a nonexistent subscription.
@@ -260,7 +262,7 @@ class Proxy(proxy.Proxy):
             :class:`~openstack.message.v2.claim.Claim` instance.
 
         :returns: One :class:`~openstack.message.v2.claim.Claim`
-        :raises: :class:`~openstack.exceptions.ResourceNotFound` when no
+        :raises: :class:`~openstack.exceptions.NotFoundException` when no
             claim matching the criteria could be found.
         """
         return self._get(_claim.Claim, claim, queue_name=queue_name)
@@ -289,7 +291,7 @@ class Proxy(proxy.Proxy):
         :param claim: The value can be either the ID of a claim or a
             :class:`~openstack.message.v2.claim.Claim` instance.
         :param bool ignore_missing: When set to ``False``
-            :class:`~openstack.exceptions.ResourceNotFound` will be
+            :class:`~openstack.exceptions.NotFoundException` will be
             raised when the claim does not exist.
             When set to ``True``, no exception will be thrown when
             attempting to delete a nonexistent claim.
@@ -302,3 +304,66 @@ class Proxy(proxy.Proxy):
             queue_name=queue_name,
             ignore_missing=ignore_missing,
         )
+
+    # ========== Utilities ==========
+
+    def wait_for_status(
+        self,
+        res: resource.ResourceT,
+        status: str,
+        failures: ty.Optional[list[str]] = None,
+        interval: ty.Union[int, float, None] = 2,
+        wait: ty.Optional[int] = None,
+        attribute: str = 'status',
+        callback: ty.Optional[ty.Callable[[int], None]] = None,
+    ) -> resource.ResourceT:
+        """Wait for the resource to be in a particular status.
+
+        :param session: The session to use for making this request.
+        :param resource: The resource to wait on to reach the status. The
+            resource must have a status attribute specified via ``attribute``.
+        :param status: Desired status of the resource.
+        :param failures: Statuses that would indicate the transition
+            failed such as 'ERROR'. Defaults to ['ERROR'].
+        :param interval: Number of seconds to wait between checks.
+        :param wait: Maximum number of seconds to wait for transition.
+            Set to ``None`` to wait forever.
+        :param attribute: Name of the resource attribute that contains the
+            status.
+        :param callback: A callback function. This will be called with a single
+            value, progress. This is API specific but is generally a percentage
+            value from 0-100.
+
+        :return: The updated resource.
+        :raises: :class:`~openstack.exceptions.ResourceTimeout` if the
+            transition to status failed to occur in ``wait`` seconds.
+        :raises: :class:`~openstack.exceptions.ResourceFailure` if the resource
+            transitioned to one of the states in ``failures``.
+        :raises: :class:`~AttributeError` if the resource does not have a
+            ``status`` attribute
+        """
+        return resource.wait_for_status(
+            self, res, status, failures, interval, wait, attribute, callback
+        )
+
+    def wait_for_delete(
+        self,
+        res: resource.ResourceT,
+        interval: int = 2,
+        wait: int = 120,
+        callback: ty.Optional[ty.Callable[[int], None]] = None,
+    ) -> resource.ResourceT:
+        """Wait for a resource to be deleted.
+
+        :param res: The resource to wait on to be deleted.
+        :param interval: Number of seconds to wait before to consecutive
+            checks.
+        :param wait: Maximum number of seconds to wait before the change.
+        :param callback: A callback function. This will be called with a single
+            value, progress, which is a percentage value from 0-100.
+
+        :returns: The resource is returned on success.
+        :raises: :class:`~openstack.exceptions.ResourceTimeout` if transition
+            to delete failed to occur in the specified seconds.
+        """
+        return resource.wait_for_delete(self, res, interval, wait, callback)
