@@ -12,11 +12,13 @@
 
 from calendar import timegm
 import collections
+import functools
 from hashlib import sha1
 import hmac
 import json
 import os
 import time
+import typing as ty
 from urllib import parse
 
 from openstack import _log
@@ -27,6 +29,7 @@ from openstack.object_store.v1 import container as _container
 from openstack.object_store.v1 import info as _info
 from openstack.object_store.v1 import obj as _obj
 from openstack import proxy
+from openstack import resource
 from openstack import utils
 
 DEFAULT_OBJECT_SEGMENT_SIZE = 1073741824  # 1GB
@@ -55,6 +58,7 @@ class Proxy(proxy.Proxy):
 
     log = _log.setup_logging('openstack')
 
+    @functools.lru_cache(maxsize=256)
     def _extract_name(self, url, service_type=None, project_id=None):
         url_path = parse.urlparse(url).path.strip()
         # Remove / from the beginning to keep the list indexes of interesting
@@ -156,7 +160,7 @@ class Proxy(proxy.Proxy):
         :param container: The value can be either the name of a container or a
             :class:`~openstack.object_store.v1.container.Container` instance.
         :param bool ignore_missing: When set to ``False``
-            :class:`~openstack.exceptions.ResourceNotFound` will be raised when
+            :class:`~openstack.exceptions.NotFoundException` will be raised when
             the container does not exist. When set to ``True``, no exception
             will be set when attempting to delete a nonexistent server.
 
@@ -173,7 +177,7 @@ class Proxy(proxy.Proxy):
             :class:`~openstack.object_store.v1.container.Container` instance.
 
         :returns: One :class:`~openstack.object_store.v1.container.Container`
-        :raises: :class:`~openstack.exceptions.ResourceNotFound`
+        :raises: :class:`~openstack.exceptions.NotFoundException`
             when no resource can be found.
         """
         return self._head(_container.Container, container)
@@ -278,7 +282,7 @@ class Proxy(proxy.Proxy):
 
         :returns: Instance of the
             :class:`~openstack.object_store.v1.obj.Object` objects.
-        :raises: :class:`~openstack.exceptions.ResourceNotFound`
+        :raises: :class:`~openstack.exceptions.NotFoundException`
             when no resource can be found.
         """
         container_name = self._get_container_name(obj=obj, container=container)
@@ -322,7 +326,7 @@ class Proxy(proxy.Proxy):
         :param container: The value can be the name of a container or a
             :class:`~openstack.object_store.v1.container.Container` instance.
 
-        :raises: :class:`~openstack.exceptions.ResourceNotFound`
+        :raises: :class:`~openstack.exceptions.NotFoundException`
             when no resource can be found.
         """
         container_name = self._get_container_name(obj=obj, container=container)
@@ -339,7 +343,7 @@ class Proxy(proxy.Proxy):
         :param container: The value can be the name of a container or a
             :class:`~openstack.object_store.v1.container.Container` instance.
 
-        :raises: :class:`~openstack.exceptions.ResourceNotFound`
+        :raises: :class:`~openstack.exceptions.NotFoundException`
             when no resource can be found.
         :returns: An iterator that iterates over chunk_size bytes
         """
@@ -427,9 +431,7 @@ class Proxy(proxy.Proxy):
             metadata[self._connection._OBJECT_SHA256_KEY] = sha256
 
         container_name = self._get_container_name(container=container)
-        endpoint = '{container}/{name}'.format(
-            container=container_name, name=name
-        )
+        endpoint = f'{container_name}/{name}'
 
         if data is not None:
             self.log.debug(
@@ -490,7 +492,7 @@ class Proxy(proxy.Proxy):
         :param container: The value can be the ID of a container or a
             :class:`~openstack.object_store.v1.container.Container` instance.
         :param bool ignore_missing: When set to ``False``
-            :class:`~openstack.exceptions.ResourceNotFound` will be raised when
+            :class:`~openstack.exceptions.NotFoundException` will be raised when
             the object does not exist.  When set to ``True``, no exception will
             be set when attempting to delete a nonexistent server.
 
@@ -514,7 +516,7 @@ class Proxy(proxy.Proxy):
             :class:`~openstack.object_store.v1.container.Container` instance.
 
         :returns: One :class:`~openstack.object_store.v1.obj.Object`
-        :raises: :class:`~openstack.exceptions.ResourceNotFound`
+        :raises: :class:`~openstack.exceptions.NotFoundException`
             when no resource can be found.
         """
         container_name = self._get_container_name(obj, container)
@@ -579,9 +581,7 @@ class Proxy(proxy.Proxy):
             metadata = self.get_object_metadata(name, container).metadata
         except exceptions.NotFoundException:
             self._connection.log.debug(
-                "swift stale check, no object: {container}/{name}".format(
-                    container=container, name=name
-                )
+                f"swift stale check, no object: {container}/{name}"
             )
             return True
 
@@ -605,7 +605,7 @@ class Proxy(proxy.Proxy):
         if not up_to_date:
             self._connection.log.debug(
                 "swift checksum mismatch: "
-                " %(filename)s!=%(container)s/%(name)s",
+                "%(filename)s!=%(container)s/%(name)s",
                 {'filename': filename, 'container': container, 'name': name},
             )
             return True
@@ -651,7 +651,7 @@ class Proxy(proxy.Proxy):
                     # While Object Storage usually expects the name to be
                     # urlencoded in most requests, the SLO manifest requires
                     # plain object names instead.
-                    path='/{name}'.format(name=parse.unquote(name)),
+                    path=f'/{parse.unquote(name)}',
                     size_bytes=segment.length,
                 )
             )
@@ -755,9 +755,7 @@ class Proxy(proxy.Proxy):
                 offset,
                 segment_size if segment_size < remaining else remaining,
             )
-            name = '{endpoint}/{index:0>6}'.format(
-                endpoint=endpoint, index=index
-            )
+            name = f'{endpoint}/{index:0>6}'
             segments[name] = segment
         return segments
 
@@ -807,7 +805,7 @@ class Proxy(proxy.Proxy):
                 continue
             name = self._object_name_from_url(result.url)
             for entry in manifest:
-                if entry['path'] == '/{name}'.format(name=parse.unquote(name)):
+                if entry['path'] == f'/{parse.unquote(name)}':
                     entry['etag'] = result.headers['Etag']
 
     def get_info(self):
@@ -875,8 +873,8 @@ class Proxy(proxy.Proxy):
             temp_url_key = self.get_temp_url_key(container)
         if not temp_url_key:
             raise exceptions.SDKException(
-                'temp_url_key was not given, nor was a temporary url key'
-                ' found for the account or the container.'
+                'temp_url_key was not given, nor was a temporary url key '
+                'found for the account or the container.'
             )
         return temp_url_key
 
@@ -928,15 +926,15 @@ class Proxy(proxy.Proxy):
 
         res = self._get_resource(_container.Container, container)
         endpoint = parse.urlparse(self.get_endpoint())
-        path = '/'.join([endpoint.path, res.name, object_prefix])
+        if isinstance(endpoint.path, bytes):
+            # To keep mypy happy: the output type will be the same as the input
+            # type
+            path = endpoint.path.decode()
+        else:
+            path = endpoint.path
+        path = '/'.join([path, res.name, object_prefix])
 
-        data = '%s\n%s\n%s\n%s\n%s' % (
-            path,
-            redirect_url,
-            max_file_size,
-            max_upload_count,
-            expires,
-        )
+        data = f'{path}\n{redirect_url}\n{max_file_size}\n{max_upload_count}\n{expires}'
         sig = hmac.new(temp_url_key, data.encode(), sha1).hexdigest()
 
         return (expires, sig)
@@ -1046,7 +1044,7 @@ class Proxy(proxy.Proxy):
                 method.upper(),
             )
 
-        expiration: float | int
+        expiration: ty.Union[float, int]
         if not absolute:
             expiration = _get_expiration(timestamp)
         else:
@@ -1064,9 +1062,9 @@ class Proxy(proxy.Proxy):
                     ip_range = ip_range.decode('utf-8')
                 except UnicodeDecodeError:
                     raise ValueError('ip_range must be representable as UTF-8')
-            hmac_parts.insert(0, "ip=%s" % ip_range)
+            hmac_parts.insert(0, f"ip={ip_range}")
 
-        hmac_body = u'\n'.join(hmac_parts)
+        hmac_body = '\n'.join(hmac_parts)
 
         temp_url_key = self._check_temp_url_key(temp_url_key=temp_url_key)
 
@@ -1081,17 +1079,13 @@ class Proxy(proxy.Proxy):
         else:
             exp = str(expiration)
 
-        temp_url = u'{path}?temp_url_sig={sig}&temp_url_expires={exp}'.format(
-            path=path_for_body,
-            sig=sig,
-            exp=exp,
-        )
+        temp_url = f'{path_for_body}?temp_url_sig={sig}&temp_url_expires={exp}'
 
         if ip_range:
-            temp_url += u'&temp_url_ip_range={}'.format(ip_range)
+            temp_url += f'&temp_url_ip_range={ip_range}'
 
         if prefix:
-            temp_url += u'&temp_url_prefix={}'.format(parts[4])
+            temp_url += f'&temp_url_prefix={parts[4]}'
         # Have return type match path from caller
         if isinstance(path, bytes):
             return temp_url.encode('utf-8')
@@ -1129,6 +1123,69 @@ class Proxy(proxy.Proxy):
                 deleted = True
         return deleted
 
+    # ========== Utilities ==========
+
+    def wait_for_status(
+        self,
+        res: resource.ResourceT,
+        status: str,
+        failures: ty.Optional[list[str]] = None,
+        interval: ty.Union[int, float, None] = 2,
+        wait: ty.Optional[int] = None,
+        attribute: str = 'status',
+        callback: ty.Optional[ty.Callable[[int], None]] = None,
+    ) -> resource.ResourceT:
+        """Wait for the resource to be in a particular status.
+
+        :param session: The session to use for making this request.
+        :param resource: The resource to wait on to reach the status. The
+            resource must have a status attribute specified via ``attribute``.
+        :param status: Desired status of the resource.
+        :param failures: Statuses that would indicate the transition
+            failed such as 'ERROR'. Defaults to ['ERROR'].
+        :param interval: Number of seconds to wait between checks.
+        :param wait: Maximum number of seconds to wait for transition.
+            Set to ``None`` to wait forever.
+        :param attribute: Name of the resource attribute that contains the
+            status.
+        :param callback: A callback function. This will be called with a single
+            value, progress. This is API specific but is generally a percentage
+            value from 0-100.
+
+        :return: The updated resource.
+        :raises: :class:`~openstack.exceptions.ResourceTimeout` if the
+            transition to status failed to occur in ``wait`` seconds.
+        :raises: :class:`~openstack.exceptions.ResourceFailure` if the resource
+            transitioned to one of the states in ``failures``.
+        :raises: :class:`~AttributeError` if the resource does not have a
+            ``status`` attribute
+        """
+        return resource.wait_for_status(
+            self, res, status, failures, interval, wait, attribute, callback
+        )
+
+    def wait_for_delete(
+        self,
+        res: resource.ResourceT,
+        interval: int = 2,
+        wait: int = 120,
+        callback: ty.Optional[ty.Callable[[int], None]] = None,
+    ) -> resource.ResourceT:
+        """Wait for a resource to be deleted.
+
+        :param res: The resource to wait on to be deleted.
+        :param interval: Number of seconds to wait before to consecutive
+            checks.
+        :param wait: Maximum number of seconds to wait before the change.
+        :param callback: A callback function. This will be called with a single
+            value, progress, which is a percentage value from 0-100.
+
+        :returns: The resource is returned on success.
+        :raises: :class:`~openstack.exceptions.ResourceTimeout` if transition
+            to delete failed to occur in the specified seconds.
+        """
+        return resource.wait_for_delete(self, res, interval, wait, callback)
+
     # ========== Project Cleanup ==========
     def _get_cleanup_dependencies(self):
         return {'object_store': {'before': []}}
