@@ -10,6 +10,8 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+import typing as ty
+
 from openstack.placement.v1 import resource_class as _resource_class
 from openstack.placement.v1 import resource_provider as _resource_provider
 from openstack.placement.v1 import (
@@ -48,7 +50,7 @@ class Proxy(proxy.Proxy):
             :class:`~openstack.placement.v1.resource_class.ResourceClass`,
             instance.
         :param bool ignore_missing: When set to ``False``
-            :class:`~openstack.exceptions.ResourceNotFound` will be raised when
+            :class:`~openstack.exceptions.NotFoundException` will be raised when
             the resource class does not exist. When set to ``True``, no
             exception will be set when attempting to delete a nonexistent
             resource class.
@@ -90,7 +92,7 @@ class Proxy(proxy.Proxy):
 
         :returns: An instance of
             :class:`~openstack.placement.v1.resource_class.ResourceClass`
-        :raises: :class:`~openstack.exceptions.ResourceNotFound` when no
+        :raises: :class:`~openstack.exceptions.NotFoundException` when no
             resource class matching the criteria could be found.
         """
         return self._get(
@@ -130,7 +132,7 @@ class Proxy(proxy.Proxy):
             :class:`~openstack.placement.v1.resource_provider.ResourceProvider`,
             instance.
         :param bool ignore_missing: When set to ``False``
-            :class:`~openstack.exceptions.ResourceNotFound` will be raised when
+            :class:`~openstack.exceptions.NotFoundException` will be raised when
             the resource provider does not exist. When set to ``True``, no
             exception will be set when attempting to delete a nonexistent
             resource provider.
@@ -172,7 +174,7 @@ class Proxy(proxy.Proxy):
 
         :returns: An instance of
             :class:`~openstack.placement.v1.resource_provider.ResourceProvider`
-        :raises: :class:`~openstack.exceptions.ResourceNotFound` when no
+        :raises: :class:`~openstack.exceptions.NotFoundException` when no
             resource provider matching the criteria could be found.
         """
         return self._get(
@@ -185,13 +187,13 @@ class Proxy(proxy.Proxy):
 
         :param name_or_id: The name or ID of a resource provider.
         :param bool ignore_missing: When set to ``False``
-            :class:`~openstack.exceptions.ResourceNotFound` will be raised when
+            :class:`~openstack.exceptions.NotFoundException` will be raised when
             the resource does not exist.  When set to ``True``, None will be
             returned when attempting to find a nonexistent resource.
 
         :returns: An instance of
             :class:`~openstack.placement.v1.resource_provider.ResourceProvider`
-        :raises: :class:`~openstack.exceptions.ResourceNotFound` when no
+        :raises: :class:`~openstack.exceptions.NotFoundException` when no
             resource provider matching the criteria could be found.
         """
         return self._find(
@@ -223,7 +225,7 @@ class Proxy(proxy.Proxy):
         :returns: An instance of
             :class:`~openstack.placement.v1.resource_provider.ResourceProvider`
             with the ``aggregates`` attribute populated.
-        :raises: :class:`~openstack.exceptions.ResourceNotFound` when no
+        :raises: :class:`~openstack.exceptions.NotFoundException` when no
             resource provider matching the criteria could be found.
         """
         res = self._get_resource(
@@ -245,7 +247,7 @@ class Proxy(proxy.Proxy):
         :returns: An instance of
             :class:`~openstack.placement.v1.resource_provider.ResourceProvider`
             with the ``aggregates`` attribute populated with the updated value.
-        :raises: :class:`~openstack.exceptions.ResourceNotFound` when no
+        :raises: :class:`~openstack.exceptions.NotFoundException` when no
             resource provider matching the criteria could be found.
         """
         res = self._get_resource(
@@ -305,7 +307,7 @@ class Proxy(proxy.Proxy):
             instance. This value must be specified when
             ``resource_provider_inventory`` is an ID.
         :param bool ignore_missing: When set to ``False``
-            :class:`~openstack.exceptions.ResourceNotFound` will be raised when
+            :class:`~openstack.exceptions.NotFoundException` will be raised when
             the resource provider inventory does not exist. When set to
             ``True``, no exception will be set when attempting to delete a
             nonexistent resource provider inventory.
@@ -379,7 +381,7 @@ class Proxy(proxy.Proxy):
 
         :returns: An instance of
             :class:`~openstack.placement.v1.resource_provider_inventory.ResourceProviderInventory`
-        :raises: :class:`~openstack.exceptions.ResourceNotFound` when no
+        :raises: :class:`~openstack.exceptions.NotFoundException` when no
             resource provider inventory matching the criteria could be found.
         """
         resource_provider_id = self._get_uri_attribute(
@@ -429,7 +431,7 @@ class Proxy(proxy.Proxy):
         :param trait: The value can be either the ID of a trait or an
             :class:`~openstack.placement.v1.trait.Trait`, instance.
         :param bool ignore_missing: When set to ``False``
-            :class:`~openstack.exceptions.ResourceNotFound` will be raised when
+            :class:`~openstack.exceptions.NotFoundException` will be raised when
             the resource provider inventory does not exist. When set to
             ``True``, no exception will be set when attempting to delete a
             nonexistent resource provider inventory.
@@ -446,7 +448,7 @@ class Proxy(proxy.Proxy):
 
         :returns: An instance of
             :class:`~openstack.placement.v1.resource_provider_inventory.ResourceProviderInventory`
-        :raises: :class:`~openstack.exceptions.ResourceNotFound` when no
+        :raises: :class:`~openstack.exceptions.NotFoundException` when no
             trait matching the criteria could be found.
         """
         return self._get(_trait.Trait, trait)
@@ -460,3 +462,66 @@ class Proxy(proxy.Proxy):
         :returns: A generator of trait objects
         """
         return self._list(_trait.Trait, **query)
+
+    # ========== Utilities ==========
+
+    def wait_for_status(
+        self,
+        res: resource.ResourceT,
+        status: str,
+        failures: ty.Optional[list[str]] = None,
+        interval: ty.Union[int, float, None] = 2,
+        wait: ty.Optional[int] = None,
+        attribute: str = 'status',
+        callback: ty.Optional[ty.Callable[[int], None]] = None,
+    ) -> resource.ResourceT:
+        """Wait for the resource to be in a particular status.
+
+        :param session: The session to use for making this request.
+        :param resource: The resource to wait on to reach the status. The
+            resource must have a status attribute specified via ``attribute``.
+        :param status: Desired status of the resource.
+        :param failures: Statuses that would indicate the transition
+            failed such as 'ERROR'. Defaults to ['ERROR'].
+        :param interval: Number of seconds to wait between checks.
+        :param wait: Maximum number of seconds to wait for transition.
+            Set to ``None`` to wait forever.
+        :param attribute: Name of the resource attribute that contains the
+            status.
+        :param callback: A callback function. This will be called with a single
+            value, progress. This is API specific but is generally a percentage
+            value from 0-100.
+
+        :return: The updated resource.
+        :raises: :class:`~openstack.exceptions.ResourceTimeout` if the
+            transition to status failed to occur in ``wait`` seconds.
+        :raises: :class:`~openstack.exceptions.ResourceFailure` if the resource
+            transitioned to one of the states in ``failures``.
+        :raises: :class:`~AttributeError` if the resource does not have a
+            ``status`` attribute
+        """
+        return resource.wait_for_status(
+            self, res, status, failures, interval, wait, attribute, callback
+        )
+
+    def wait_for_delete(
+        self,
+        res: resource.ResourceT,
+        interval: int = 2,
+        wait: int = 120,
+        callback: ty.Optional[ty.Callable[[int], None]] = None,
+    ) -> resource.ResourceT:
+        """Wait for a resource to be deleted.
+
+        :param res: The resource to wait on to be deleted.
+        :param interval: Number of seconds to wait before to consecutive
+            checks.
+        :param wait: Maximum number of seconds to wait before the change.
+        :param callback: A callback function. This will be called with a single
+            value, progress, which is a percentage value from 0-100.
+
+        :returns: The resource is returned on success.
+        :raises: :class:`~openstack.exceptions.ResourceTimeout` if transition
+            to delete failed to occur in the specified seconds.
+        """
+        return resource.wait_for_delete(self, res, interval, wait, callback)
