@@ -9,6 +9,9 @@
 # WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 # License for the specific language governing permissions and limitations
 # under the License.
+
+import typing as ty
+
 from openstack import exceptions
 from openstack import proxy
 from openstack import resource
@@ -16,6 +19,7 @@ from openstack.shared_file_system.v2 import (
     availability_zone as _availability_zone,
 )
 from openstack.shared_file_system.v2 import limit as _limit
+from openstack.shared_file_system.v2 import quota_class_set as _quota_class_set
 from openstack.shared_file_system.v2 import resource_locks as _resource_locks
 from openstack.shared_file_system.v2 import share as _share
 from openstack.shared_file_system.v2 import share_group as _share_group
@@ -58,6 +62,7 @@ class Proxy(proxy.Proxy):
         "share_group": _share_group.ShareGroup,
         "share_group_snapshot": _share_group_snapshot.ShareGroupSnapshot,
         "resource_locks": _resource_locks.ResourceLock,
+        "quota_class_set": _quota_class_set.QuotaClassSet,
     }
 
     def availability_zones(self):
@@ -122,7 +127,7 @@ class Proxy(proxy.Proxy):
 
         :param name_or_id: The name or ID of a share.
         :param bool ignore_missing: When set to ``False``
-            :class:`~openstack.exceptions.ResourceNotFound` will be
+            :class:`~openstack.exceptions.NotFoundException` will be
             raised when the resource does not exist.
             When set to ``True``, None will be returned when
             attempting to find a nonexistent resource.
@@ -310,7 +315,7 @@ class Proxy(proxy.Proxy):
 
         :param name_or_id: The name or ID of a share group.
         :param bool ignore_missing: When set to ``False``
-            :class:`~openstack.exceptions.ResourceNotFound` will be
+            :class:`~openstack.exceptions.NotFoundException` will be
             raised when the resource does not exist.
             When set to ``True``, None will be returned when
             attempting to find a nonexistent resource.
@@ -355,47 +360,6 @@ class Proxy(proxy.Proxy):
             ignore_missing=ignore_missing,
         )
 
-    def wait_for_status(
-        self,
-        res,
-        status='active',
-        failures=None,
-        interval=2,
-        wait=120,
-        status_attr_name='status',
-    ):
-        """Wait for a resource to be in a particular status.
-        :param res: The resource to wait on to reach the specified status.
-            The resource must have a ``status`` attribute.
-        :type resource: A :class:`~openstack.resource.Resource` object.
-        :param status: Desired status.
-        :param failures: Statuses that would be interpreted as failures.
-        :type failures: :py:class:`list`
-        :param interval: Number of seconds to wait before to consecutive
-            checks. Default to 2.
-        :param wait: Maximum number of seconds to wait before the change.
-            Default to 120.
-        :param status_attr_name: name of the attribute to reach the desired
-            status.
-        :returns: The resource is returned on success.
-        :raises: :class:`~openstack.exceptions.ResourceTimeout` if transition
-            to the desired status failed to occur in specified seconds.
-        :raises: :class:`~openstack.exceptions.ResourceFailure` if the resource
-            has transited to one of the failure statuses.
-        :raises: :class:`~AttributeError` if the resource does not have a
-            ``status`` attribute.
-        """
-        failures = [] if failures is None else failures
-        return resource.wait_for_status(
-            self,
-            res,
-            status,
-            failures,
-            interval,
-            wait,
-            attribute=status_attr_name,
-        )
-
     def storage_pools(self, details=True, **query):
         """Lists all back-end storage pools with details
 
@@ -594,7 +558,7 @@ class Proxy(proxy.Proxy):
         return self._create(
             _share_network_subnet.ShareNetworkSubnet,
             **attrs,
-            share_network_id=share_network_id
+            share_network_id=share_network_id,
         )
 
     def delete_share_network_subnet(
@@ -617,21 +581,6 @@ class Proxy(proxy.Proxy):
             ignore_missing=ignore_missing,
         )
 
-    def wait_for_delete(self, res, interval=2, wait=120):
-        """Wait for a resource to be deleted.
-
-        :param res: The resource to wait on to be deleted.
-        :type resource: A :class:`~openstack.resource.Resource` object.
-        :param interval: Number of seconds to wait before to consecutive
-            checks. Default to 2.
-        :param wait: Maximum number of seconds to wait before the change.
-            Default to 120.
-        :returns: The resource is returned on success.
-        :raises: :class:`~openstack.exceptions.ResourceTimeout` if transition
-            to delete failed to occur in the specified seconds.
-        """
-        return resource.wait_for_delete(self, res, interval, wait)
-
     def share_snapshot_instances(self, details=True, **query):
         """Lists all share snapshot instances with details.
 
@@ -654,7 +603,7 @@ class Proxy(proxy.Proxy):
         return self._list(
             _share_snapshot_instance.ShareSnapshotInstance,
             base_path=base_path,
-            **query
+            **query,
         )
 
     def get_share_snapshot_instance(self, snapshot_instance_id):
@@ -857,7 +806,7 @@ class Proxy(proxy.Proxy):
         :rtype: :class:`~openstack.shared_file_system.v2.
             share_access_rules.ShareAccessRules`
         """
-        base_path = "/shares/%s/action" % (share_id,)
+        base_path = f"/shares/{share_id}/action"
         return self._create(
             _share_access_rule.ShareAccessRule, base_path=base_path, **attrs
         )
@@ -874,13 +823,18 @@ class Proxy(proxy.Proxy):
         :rtype: ``requests.models.Response`` HTTP response from internal
             requests client
         """
-        res = self._get_resource(_share_access_rule.ShareAccessRule, access_id)
-        return res.delete(
-            self,
-            share_id,
-            ignore_missing=ignore_missing,
-            unrestrict=unrestrict,
+        res = self._get_resource(
+            _share_access_rule.ShareAccessRule, access_id, share_id=share_id
         )
+        try:
+            return res.delete(
+                self,
+                unrestrict=unrestrict,
+            )
+        except exceptions.NotFoundException:
+            if ignore_missing:
+                return None
+            raise
 
     def share_group_snapshots(self, details=True, **query):
         """Lists all share group snapshots.
@@ -911,7 +865,7 @@ class Proxy(proxy.Proxy):
         return self._list(
             _share_group_snapshot.ShareGroupSnapshot,
             base_path=base_path,
-            **query
+            **query,
         )
 
     def share_group_snapshot_members(self, group_snapshot_id):
@@ -956,7 +910,7 @@ class Proxy(proxy.Proxy):
         return self._create(
             _share_group_snapshot.ShareGroupSnapshot,
             share_group_id=share_group_id,
-            **attrs
+            **attrs,
         )
 
     def reset_share_group_snapshot_status(self, group_snapshot_id, status):
@@ -985,7 +939,7 @@ class Proxy(proxy.Proxy):
         return self._update(
             _share_group_snapshot.ShareGroupSnapshot,
             group_snapshot_id,
-            **attrs
+            **attrs,
         )
 
     def delete_share_group_snapshot(
@@ -1087,7 +1041,7 @@ class Proxy(proxy.Proxy):
                 keys_failed_to_delete.append(key)
         if keys_failed_to_delete:
             raise exceptions.SDKException(
-                "Some keys failed to be deleted %s" % keys_failed_to_delete
+                f"Some keys failed to be deleted {keys_failed_to_delete}"
             )
 
     def resource_locks(self, **query):
@@ -1127,6 +1081,15 @@ class Proxy(proxy.Proxy):
         :rtype: :class:`~openstack.shared_file_system.v2.
             resource_locks.ResourceLock`
         """
+
+        if query.get('resource_type'):
+            # The _create method has a parameter named resource_type, which
+            # refers to the type of resource to be created, so we need to avoid
+            # a conflict of parameters we are sending to the method.
+            query['__conflicting_attrs'] = {
+                'resource_type': query.get('resource_type')
+            }
+            query.pop('resource_type')
         return self._list(_resource_locks.ResourceLock, **query)
 
     def get_resource_lock(self, resource_lock):
@@ -1198,3 +1161,89 @@ class Proxy(proxy.Proxy):
             }
             attrs.pop('resource_type')
         return self._create(_resource_locks.ResourceLock, **attrs)
+
+    def get_quota_class_set(self, quota_class_name):
+        """Get quota class set.
+
+        :param quota_class_name: The name of the quota class
+        :returns: A :class:`~openstack.shared_file_system.v2
+            .quota_class_set.QuotaClassSet`
+        """
+        return self._get(_quota_class_set.QuotaClassSet, quota_class_name)
+
+    def update_quota_class_set(self, quota_class_name, **attrs):
+        """Update quota class set.
+
+        :param quota_class_name: The name of the quota class
+        :param attrs: The attributes to update on the quota class set
+        :returns: the updated quota class set
+        :rtype: :class:`~openstack.shared_file_system.v2
+            .quota_class_set.QuotaClassSet`
+        """
+
+        return self._update(
+            _quota_class_set.QuotaClassSet, quota_class_name, **attrs
+        )
+
+    # ========== Utilities ==========
+
+    def wait_for_status(
+        self,
+        res: resource.ResourceT,
+        status: str,
+        failures: ty.Optional[list[str]] = None,
+        interval: ty.Union[int, float, None] = 2,
+        wait: ty.Optional[int] = None,
+        attribute: str = 'status',
+        callback: ty.Optional[ty.Callable[[int], None]] = None,
+    ) -> resource.ResourceT:
+        """Wait for the resource to be in a particular status.
+
+        :param session: The session to use for making this request.
+        :param resource: The resource to wait on to reach the status. The
+            resource must have a status attribute specified via ``attribute``.
+        :param status: Desired status of the resource.
+        :param failures: Statuses that would indicate the transition
+            failed such as 'ERROR'. Defaults to ['ERROR'].
+        :param interval: Number of seconds to wait between checks.
+        :param wait: Maximum number of seconds to wait for transition.
+            Set to ``None`` to wait forever.
+        :param attribute: Name of the resource attribute that contains the
+            status.
+        :param callback: A callback function. This will be called with a single
+            value, progress. This is API specific but is generally a percentage
+            value from 0-100.
+
+        :return: The updated resource.
+        :raises: :class:`~openstack.exceptions.ResourceTimeout` if the
+            transition to status failed to occur in ``wait`` seconds.
+        :raises: :class:`~openstack.exceptions.ResourceFailure` if the resource
+            transitioned to one of the states in ``failures``.
+        :raises: :class:`~AttributeError` if the resource does not have a
+            ``status`` attribute
+        """
+        return resource.wait_for_status(
+            self, res, status, failures, interval, wait, attribute, callback
+        )
+
+    def wait_for_delete(
+        self,
+        res: resource.ResourceT,
+        interval: int = 2,
+        wait: int = 120,
+        callback: ty.Optional[ty.Callable[[int], None]] = None,
+    ) -> resource.ResourceT:
+        """Wait for a resource to be deleted.
+
+        :param res: The resource to wait on to be deleted.
+        :param interval: Number of seconds to wait before to consecutive
+            checks.
+        :param wait: Maximum number of seconds to wait before the change.
+        :param callback: A callback function. This will be called with a single
+            value, progress, which is a percentage value from 0-100.
+
+        :returns: The resource is returned on success.
+        :raises: :class:`~openstack.exceptions.ResourceTimeout` if transition
+            to delete failed to occur in the specified seconds.
+        """
+        return resource.wait_for_delete(self, res, interval, wait, callback)
