@@ -24,7 +24,7 @@ from openstack.tests.functional import base
 
 class TestZone(base.BaseFunctionalTest):
     def setUp(self):
-        super(TestZone, self).setUp()
+        super().setUp()
         if not self.user_cloud.has_service('dns'):
             self.skipTest('dns service not supported by cloud')
 
