@@ -10,6 +10,7 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+import base64
 from unittest import mock
 
 from keystoneauth1 import adapter
@@ -82,6 +83,7 @@ FAKE = {
     "service_step": {},
     "secure_boot": True,
     "shard": "TestShard",
+    "runbook": None,
     "states": [
         {
             "href": "http://127.0.0.1:6385/v1/nodes/<NODE_ID>/states",
@@ -160,6 +162,7 @@ class TestNode(base.TestCase):
         self.assertEqual(FAKE['resource_class'], sot.resource_class)
         self.assertEqual(FAKE['service_step'], sot.service_step)
         self.assertEqual(FAKE['secure_boot'], sot.is_secure_boot)
+        self.assertEqual(FAKE['runbook'], sot.runbook)
         self.assertEqual(FAKE['states'], sot.states)
         self.assertEqual(
             FAKE['target_provision_state'], sot.target_provision_state
@@ -206,7 +209,7 @@ class TestNode(base.TestCase):
 @mock.patch.object(node.Node, 'fetch', autospec=True)
 class TestNodeWaitForProvisionState(base.TestCase):
     def setUp(self):
-        super(TestNodeWaitForProvisionState, self).setUp()
+        super().setUp()
         self.node = node.Node(**FAKE)
         self.session = mock.Mock()
 
@@ -297,7 +300,7 @@ class TestNodeWaitForProvisionState(base.TestCase):
 @mock.patch.object(exceptions, 'raise_from_response', mock.Mock())
 class TestNodeSetProvisionState(base.TestCase):
     def setUp(self):
-        super(TestNodeSetProvisionState, self).setUp()
+        super().setUp()
         self.node = node.Node(**FAKE)
         self.session = mock.Mock(
             spec=adapter.Adapter, default_microversion=None
@@ -307,7 +310,7 @@ class TestNodeSetProvisionState(base.TestCase):
         result = self.node.set_provision_state(self.session, 'active')
         self.assertIs(result, self.node)
         self.session.put.assert_called_once_with(
-            'nodes/%s/states/provision' % self.node.id,
+            f'nodes/{self.node.id}/states/provision',
             json={'target': 'active'},
             headers=mock.ANY,
             microversion=None,
@@ -318,7 +321,7 @@ class TestNodeSetProvisionState(base.TestCase):
         result = self.node.set_provision_state(self.session, 'manage')
         self.assertIs(result, self.node)
         self.session.put.assert_called_once_with(
-            'nodes/%s/states/provision' % self.node.id,
+            f'nodes/{self.node.id}/states/provision',
             json={'target': 'manage'},
             headers=mock.ANY,
             microversion='1.4',
@@ -331,20 +334,34 @@ class TestNodeSetProvisionState(base.TestCase):
         )
         self.assertIs(result, self.node)
         self.session.put.assert_called_once_with(
-            'nodes/%s/states/provision' % self.node.id,
+            f'nodes/{self.node.id}/states/provision',
             json={'target': 'active', 'configdrive': 'abcd'},
             headers=mock.ANY,
             microversion=None,
             retriable_status_codes=_common.RETRIABLE_STATUS_CODES,
         )
 
+    def test_deploy_with_configdrive_as_bytestring(self):
+        config_drive = base64.b64encode(b'foo')
+        result = self.node.set_provision_state(
+            self.session, 'active', config_drive=config_drive
+        )
+        self.assertIs(result, self.node)
+        self.session.put.assert_called_once_with(
+            f'nodes/{self.node.id}/states/provision',
+            json={'target': 'active', 'configdrive': config_drive.decode()},
+            headers=mock.ANY,
+            microversion=None,
+            retriable_status_codes=_common.RETRIABLE_STATUS_CODES,
+        )
+
     def test_rebuild_with_configdrive(self):
         result = self.node.set_provision_state(
             self.session, 'rebuild', config_drive='abcd'
         )
         self.assertIs(result, self.node)
         self.session.put.assert_called_once_with(
-            'nodes/%s/states/provision' % self.node.id,
+            f'nodes/{self.node.id}/states/provision',
             json={'target': 'rebuild', 'configdrive': 'abcd'},
             headers=mock.ANY,
             microversion='1.35',
@@ -359,7 +376,7 @@ class TestNodeSetProvisionState(base.TestCase):
             )
             self.assertIs(result, self.node)
             self.session.put.assert_called_once_with(
-                'nodes/%s/states/provision' % self.node.id,
+                f'nodes/{self.node.id}/states/provision',
                 json={'target': target, 'configdrive': {'user_data': 'abcd'}},
                 headers=mock.ANY,
                 microversion='1.56',
@@ -374,7 +391,7 @@ class TestNodeSetProvisionState(base.TestCase):
 
         self.assertIs(result, self.node)
         self.session.put.assert_called_once_with(
-            'nodes/%s/states/provision' % self.node.id,
+            f'nodes/{self.node.id}/states/provision',
             json={'target': 'active', 'deploy_steps': deploy_steps},
             headers=mock.ANY,
             microversion='1.69',
@@ -389,7 +406,7 @@ class TestNodeSetProvisionState(base.TestCase):
 
         self.assertIs(result, self.node)
         self.session.put.assert_called_once_with(
-            'nodes/%s/states/provision' % self.node.id,
+            f'nodes/{self.node.id}/states/provision',
             json={'target': 'rebuild', 'deploy_steps': deploy_steps},
             headers=mock.ANY,
             microversion='1.69',
@@ -401,7 +418,7 @@ class TestNodeSetProvisionState(base.TestCase):
 
         self.assertIs(result, self.node)
         self.session.put.assert_called_once_with(
-            'nodes/%s/states/provision' % self.node.id,
+            f'nodes/{self.node.id}/states/provision',
             json={'target': 'unhold'},
             headers=mock.ANY,
             microversion='1.85',
@@ -416,20 +433,50 @@ class TestNodeSetProvisionState(base.TestCase):
 
         self.assertIs(result, self.node)
         self.session.put.assert_called_once_with(
-            'nodes/%s/states/provision' % self.node.id,
+            f'nodes/{self.node.id}/states/provision',
             json={'target': 'service', 'service_steps': service_steps},
             headers=mock.ANY,
             microversion='1.87',
             retriable_status_codes=_common.RETRIABLE_STATUS_CODES,
         )
 
+    def test_set_provision_state_clean_runbook(self):
+        runbook = 'CUSTOM_AWESOME'
+        result = self.node.set_provision_state(
+            self.session, 'clean', runbook=runbook
+        )
+
+        self.assertIs(result, self.node)
+        self.session.put.assert_called_once_with(
+            f'nodes/{self.node.id}/states/provision',
+            json={'target': 'clean', 'runbook': runbook},
+            headers=mock.ANY,
+            microversion='1.92',
+            retriable_status_codes=_common.RETRIABLE_STATUS_CODES,
+        )
+
+    def test_set_provision_state_service_runbook(self):
+        runbook = 'CUSTOM_AWESOME'
+        result = self.node.set_provision_state(
+            self.session, 'service', runbook=runbook
+        )
+
+        self.assertIs(result, self.node)
+        self.session.put.assert_called_once_with(
+            f'nodes/{self.node.id}/states/provision',
+            json={'target': 'service', 'runbook': runbook},
+            headers=mock.ANY,
+            microversion='1.92',
+            retriable_status_codes=_common.RETRIABLE_STATUS_CODES,
+        )
+
 
 @mock.patch.object(node.Node, '_translate_response', mock.Mock())
 @mock.patch.object(node.Node, '_get_session', lambda self, x: x)
 @mock.patch.object(node.Node, 'set_provision_state', autospec=True)
 class TestNodeCreate(base.TestCase):
     def setUp(self):
-        super(TestNodeCreate, self).setUp()
+        super().setUp()
         self.new_state = None
         self.session = mock.Mock(spec=adapter.Adapter)
         self.session.default_microversion = '1.1'
@@ -541,22 +588,24 @@ class TestNodeCreate(base.TestCase):
 @mock.patch.object(node.Node, '_get_session', lambda self, x: x)
 class TestNodeVif(base.TestCase):
     def setUp(self):
-        super(TestNodeVif, self).setUp()
+        super().setUp()
         self.session = mock.Mock(spec=adapter.Adapter)
-        self.session.default_microversion = '1.28'
+        self.session.default_microversion = '1.67'
         self.session.log = mock.Mock()
         self.node = node.Node(
             id='c29db401-b6a7-4530-af8e-20a720dee946', driver=FAKE['driver']
         )
         self.vif_id = '714bdf6d-2386-4b5e-bd0d-bc036f04b1ef'
+        self.vif_port_uuid = 'port-uuid'
+        self.vif_portgroup_uuid = 'portgroup-uuid'
 
     def test_attach_vif(self):
         self.assertIsNone(self.node.attach_vif(self.session, self.vif_id))
         self.session.post.assert_called_once_with(
-            'nodes/%s/vifs' % self.node.id,
+            f'nodes/{self.node.id}/vifs',
             json={'id': self.vif_id},
             headers=mock.ANY,
-            microversion='1.28',
+            microversion='1.67',
             retriable_status_codes=[409, 503],
         )
 
@@ -567,19 +616,62 @@ class TestNodeVif(base.TestCase):
             )
         )
         self.session.post.assert_called_once_with(
-            'nodes/%s/vifs' % self.node.id,
+            f'nodes/{self.node.id}/vifs',
             json={'id': self.vif_id},
             headers=mock.ANY,
-            microversion='1.28',
+            microversion='1.67',
             retriable_status_codes=[503],
         )
 
+    def test_attach_vif_with_port_uuid(self):
+        self.assertIsNone(
+            self.node.attach_vif(
+                self.session, self.vif_id, port_id=self.vif_port_uuid
+            )
+        )
+        self.session.post.assert_called_once_with(
+            f'nodes/{self.node.id}/vifs',
+            json={'id': self.vif_id, 'port_uuid': self.vif_port_uuid},
+            headers=mock.ANY,
+            microversion='1.67',
+            retriable_status_codes=[409, 503],
+        )
+
+    def test_attach_vif_with_portgroup_uuid(self):
+        self.assertIsNone(
+            self.node.attach_vif(
+                self.session,
+                self.vif_id,
+                port_group_id=self.vif_portgroup_uuid,
+            )
+        )
+        self.session.post.assert_called_once_with(
+            f'nodes/{self.node.id}/vifs',
+            json={
+                'id': self.vif_id,
+                'portgroup_uuid': self.vif_portgroup_uuid,
+            },
+            headers=mock.ANY,
+            microversion='1.67',
+            retriable_status_codes=[409, 503],
+        )
+
+    def test_attach_vif_with_port_uuid_and_portgroup_uuid(self):
+        self.assertRaises(
+            exceptions.InvalidRequest,
+            self.node.attach_vif,
+            self.session,
+            self.vif_id,
+            port_id=self.vif_port_uuid,
+            port_group_id=self.vif_portgroup_uuid,
+        )
+
     def test_detach_vif_existing(self):
         self.assertTrue(self.node.detach_vif(self.session, self.vif_id))
         self.session.delete.assert_called_once_with(
-            'nodes/%s/vifs/%s' % (self.node.id, self.vif_id),
+            f'nodes/{self.node.id}/vifs/{self.vif_id}',
             headers=mock.ANY,
-            microversion='1.28',
+            microversion='1.67',
             retriable_status_codes=_common.RETRIABLE_STATUS_CODES,
         )
 
@@ -587,9 +679,9 @@ class TestNodeVif(base.TestCase):
         self.session.delete.return_value.status_code = 400
         self.assertFalse(self.node.detach_vif(self.session, self.vif_id))
         self.session.delete.assert_called_once_with(
-            'nodes/%s/vifs/%s' % (self.node.id, self.vif_id),
+            f'nodes/{self.node.id}/vifs/{self.vif_id}',
             headers=mock.ANY,
-            microversion='1.28',
+            microversion='1.67',
             retriable_status_codes=_common.RETRIABLE_STATUS_CODES,
         )
 
@@ -603,9 +695,9 @@ class TestNodeVif(base.TestCase):
         res = self.node.list_vifs(self.session)
         self.assertEqual(['1234', '5678'], res)
         self.session.get.assert_called_once_with(
-            'nodes/%s/vifs' % self.node.id,
+            f'nodes/{self.node.id}/vifs',
             headers=mock.ANY,
-            microversion='1.28',
+            microversion='1.67',
         )
 
     def test_incompatible_microversion(self):
@@ -626,12 +718,29 @@ class TestNodeVif(base.TestCase):
             exceptions.NotSupported, self.node.list_vifs, self.session
         )
 
+    def test_incompatible_microversion_optional_params(self):
+        self.session.default_microversion = '1.28'
+        self.assertRaises(
+            exceptions.NotSupported,
+            self.node.attach_vif,
+            self.session,
+            self.vif_id,
+            port_id=self.vif_port_uuid,
+        )
+        self.assertRaises(
+            exceptions.NotSupported,
+            self.node.attach_vif,
+            self.session,
+            self.vif_id,
+            port_group_id=self.vif_portgroup_uuid,
+        )
+
 
 @mock.patch.object(exceptions, 'raise_from_response', mock.Mock())
 @mock.patch.object(node.Node, '_get_session', lambda self, x: x)
 class TestNodeValidate(base.TestCase):
     def setUp(self):
-        super(TestNodeValidate, self).setUp()
+        super().setUp()
         self.session = mock.Mock(spec=adapter.Adapter)
         self.session.default_microversion = '1.28'
         self.node = node.Node(**FAKE)
@@ -690,7 +799,7 @@ class TestNodeValidate(base.TestCase):
 @mock.patch.object(node.Node, 'fetch', autospec=True)
 class TestNodeWaitForReservation(base.TestCase):
     def setUp(self):
-        super(TestNodeWaitForReservation, self).setUp()
+        super().setUp()
         self.session = mock.Mock(spec=adapter.Adapter)
         self.session.default_microversion = '1.6'
         self.session.log = mock.Mock()
@@ -740,7 +849,7 @@ class TestNodeInjectNMI(base.TestCase):
     def test_inject_nmi(self):
         self.node.inject_nmi(self.session)
         self.session.put.assert_called_once_with(
-            'nodes/%s/management/inject_nmi' % FAKE['uuid'],
+            'nodes/{}/management/inject_nmi'.format(FAKE['uuid']),
             json={},
             headers=mock.ANY,
             microversion='1.29',
@@ -760,7 +869,7 @@ class TestNodeInjectNMI(base.TestCase):
 @mock.patch.object(exceptions, 'raise_from_response', mock.Mock())
 class TestNodeSetPowerState(base.TestCase):
     def setUp(self):
-        super(TestNodeSetPowerState, self).setUp()
+        super().setUp()
         self.node = node.Node(**FAKE)
         self.session = mock.Mock(
             spec=adapter.Adapter, default_microversion=None
@@ -769,7 +878,7 @@ class TestNodeSetPowerState(base.TestCase):
     def test_power_on(self):
         self.node.set_power_state(self.session, 'power on')
         self.session.put.assert_called_once_with(
-            'nodes/%s/states/power' % FAKE['uuid'],
+            'nodes/{}/states/power'.format(FAKE['uuid']),
             json={'target': 'power on'},
             headers=mock.ANY,
             microversion=None,
@@ -779,7 +888,7 @@ class TestNodeSetPowerState(base.TestCase):
     def test_soft_power_on(self):
         self.node.set_power_state(self.session, 'soft power off')
         self.session.put.assert_called_once_with(
-            'nodes/%s/states/power' % FAKE['uuid'],
+            'nodes/{}/states/power'.format(FAKE['uuid']),
             json={'target': 'soft power off'},
             headers=mock.ANY,
             microversion='1.27',
@@ -792,7 +901,7 @@ class TestNodeSetPowerState(base.TestCase):
 @mock.patch.object(node.Node, '_get_session', lambda self, x: x)
 class TestNodeMaintenance(base.TestCase):
     def setUp(self):
-        super(TestNodeMaintenance, self).setUp()
+        super().setUp()
         self.node = node.Node.existing(**FAKE)
         self.session = mock.Mock(
             spec=adapter.Adapter,
@@ -803,7 +912,7 @@ class TestNodeMaintenance(base.TestCase):
     def test_set(self):
         self.node.set_maintenance(self.session)
         self.session.put.assert_called_once_with(
-            'nodes/%s/maintenance' % self.node.id,
+            f'nodes/{self.node.id}/maintenance',
             json={'reason': None},
             headers=mock.ANY,
             microversion=mock.ANY,
@@ -812,7 +921,7 @@ class TestNodeMaintenance(base.TestCase):
     def test_set_with_reason(self):
         self.node.set_maintenance(self.session, 'No work on Monday')
         self.session.put.assert_called_once_with(
-            'nodes/%s/maintenance' % self.node.id,
+            f'nodes/{self.node.id}/maintenance',
             json={'reason': 'No work on Monday'},
             headers=mock.ANY,
             microversion=mock.ANY,
@@ -821,7 +930,7 @@ class TestNodeMaintenance(base.TestCase):
     def test_unset(self):
         self.node.unset_maintenance(self.session)
         self.session.delete.assert_called_once_with(
-            'nodes/%s/maintenance' % self.node.id,
+            f'nodes/{self.node.id}/maintenance',
             json=None,
             headers=mock.ANY,
             microversion=mock.ANY,
@@ -831,7 +940,7 @@ class TestNodeMaintenance(base.TestCase):
         self.node.is_maintenance = True
         self.node.commit(self.session)
         self.session.put.assert_called_once_with(
-            'nodes/%s/maintenance' % self.node.id,
+            f'nodes/{self.node.id}/maintenance',
             json={'reason': None},
             headers=mock.ANY,
             microversion=mock.ANY,
@@ -844,7 +953,7 @@ class TestNodeMaintenance(base.TestCase):
         self.node.maintenance_reason = 'No work on Monday'
         self.node.commit(self.session)
         self.session.put.assert_called_once_with(
-            'nodes/%s/maintenance' % self.node.id,
+            f'nodes/{self.node.id}/maintenance',
             json={'reason': 'No work on Monday'},
             headers=mock.ANY,
             microversion=mock.ANY,
@@ -856,14 +965,14 @@ class TestNodeMaintenance(base.TestCase):
         self.node.name = 'lazy-3000'
         self.node.commit(self.session)
         self.session.put.assert_called_once_with(
-            'nodes/%s/maintenance' % self.node.id,
+            f'nodes/{self.node.id}/maintenance',
             json={'reason': None},
             headers=mock.ANY,
             microversion=mock.ANY,
         )
 
         self.session.patch.assert_called_once_with(
-            'nodes/%s' % self.node.id,
+            f'nodes/{self.node.id}',
             json=[{'path': '/name', 'op': 'replace', 'value': 'lazy-3000'}],
             headers=mock.ANY,
             microversion=mock.ANY,
@@ -875,14 +984,14 @@ class TestNodeMaintenance(base.TestCase):
         self.node.name = 'lazy-3000'
         self.node.commit(self.session)
         self.session.put.assert_called_once_with(
-            'nodes/%s/maintenance' % self.node.id,
+            f'nodes/{self.node.id}/maintenance',
             json={'reason': 'No work on Monday'},
             headers=mock.ANY,
             microversion=mock.ANY,
         )
 
         self.session.patch.assert_called_once_with(
-            'nodes/%s' % self.node.id,
+            f'nodes/{self.node.id}',
             json=[{'path': '/name', 'op': 'replace', 'value': 'lazy-3000'}],
             headers=mock.ANY,
             microversion=mock.ANY,
@@ -900,7 +1009,7 @@ class TestNodeMaintenance(base.TestCase):
         self.node.commit(self.session)
 
         self.session.put.assert_called_once_with(
-            'nodes/%s/maintenance' % self.node.id,
+            f'nodes/{self.node.id}/maintenance',
             json={'reason': 'No work on Monday'},
             headers=mock.ANY,
             microversion=mock.ANY,
@@ -911,7 +1020,7 @@ class TestNodeMaintenance(base.TestCase):
         self.assertIsNone(self.node.maintenance_reason)
 
         self.session.delete.assert_called_once_with(
-            'nodes/%s/maintenance' % self.node.id,
+            f'nodes/{self.node.id}/maintenance',
             json=None,
             headers=mock.ANY,
             microversion=mock.ANY,
@@ -931,7 +1040,7 @@ class TestNodeBootDevice(base.TestCase):
     def test_get_boot_device(self):
         self.node.get_boot_device(self.session)
         self.session.get.assert_called_once_with(
-            'nodes/%s/management/boot_device' % self.node.id,
+            f'nodes/{self.node.id}/management/boot_device',
             headers=mock.ANY,
             microversion=mock.ANY,
             retriable_status_codes=_common.RETRIABLE_STATUS_CODES,
@@ -940,7 +1049,7 @@ class TestNodeBootDevice(base.TestCase):
     def test_set_boot_device(self):
         self.node.set_boot_device(self.session, 'pxe', persistent=False)
         self.session.put.assert_called_once_with(
-            'nodes/%s/management/boot_device' % self.node.id,
+            f'nodes/{self.node.id}/management/boot_device',
             json={'boot_device': 'pxe', 'persistent': False},
             headers=mock.ANY,
             microversion=mock.ANY,
@@ -950,7 +1059,7 @@ class TestNodeBootDevice(base.TestCase):
     def test_get_supported_boot_devices(self):
         self.node.get_supported_boot_devices(self.session)
         self.session.get.assert_called_once_with(
-            'nodes/%s/management/boot_device/supported' % self.node.id,
+            f'nodes/{self.node.id}/management/boot_device/supported',
             headers=mock.ANY,
             microversion=mock.ANY,
             retriable_status_codes=_common.RETRIABLE_STATUS_CODES,
@@ -962,7 +1071,7 @@ class TestNodeBootDevice(base.TestCase):
 @mock.patch.object(exceptions, 'raise_from_response', mock.Mock())
 class TestNodeSetBootMode(base.TestCase):
     def setUp(self):
-        super(TestNodeSetBootMode, self).setUp()
+        super().setUp()
         self.node = node.Node(**FAKE)
         self.session = mock.Mock(
             spec=adapter.Adapter, default_microversion='1.1'
@@ -971,7 +1080,7 @@ class TestNodeSetBootMode(base.TestCase):
     def test_node_set_boot_mode(self):
         self.node.set_boot_mode(self.session, 'uefi')
         self.session.put.assert_called_once_with(
-            'nodes/%s/states/boot_mode' % self.node.id,
+            f'nodes/{self.node.id}/states/boot_mode',
             json={'target': 'uefi'},
             headers=mock.ANY,
             microversion=mock.ANY,
@@ -989,7 +1098,7 @@ class TestNodeSetBootMode(base.TestCase):
 @mock.patch.object(exceptions, 'raise_from_response', mock.Mock())
 class TestNodeSetSecureBoot(base.TestCase):
     def setUp(self):
-        super(TestNodeSetSecureBoot, self).setUp()
+        super().setUp()
         self.node = node.Node(**FAKE)
         self.session = mock.Mock(
             spec=adapter.Adapter, default_microversion='1.1'
@@ -998,7 +1107,7 @@ class TestNodeSetSecureBoot(base.TestCase):
     def test_node_set_secure_boot(self):
         self.node.set_secure_boot(self.session, True)
         self.session.put.assert_called_once_with(
-            'nodes/%s/states/secure_boot' % self.node.id,
+            f'nodes/{self.node.id}/states/secure_boot',
             json={'target': True},
             headers=mock.ANY,
             microversion=mock.ANY,
@@ -1016,7 +1125,7 @@ class TestNodeSetSecureBoot(base.TestCase):
 @mock.patch.object(exceptions, 'raise_from_response', mock.Mock())
 class TestNodeTraits(base.TestCase):
     def setUp(self):
-        super(TestNodeTraits, self).setUp()
+        super().setUp()
         self.node = node.Node(**FAKE)
         self.session = mock.Mock(
             spec=adapter.Adapter, default_microversion='1.37'
@@ -1026,7 +1135,7 @@ class TestNodeTraits(base.TestCase):
     def test_node_add_trait(self):
         self.node.add_trait(self.session, 'CUSTOM_FAKE')
         self.session.put.assert_called_once_with(
-            'nodes/%s/traits/%s' % (self.node.id, 'CUSTOM_FAKE'),
+            'nodes/{}/traits/{}'.format(self.node.id, 'CUSTOM_FAKE'),
             json=None,
             headers=mock.ANY,
             microversion='1.37',
@@ -1036,7 +1145,7 @@ class TestNodeTraits(base.TestCase):
     def test_remove_trait(self):
         self.assertTrue(self.node.remove_trait(self.session, 'CUSTOM_FAKE'))
         self.session.delete.assert_called_once_with(
-            'nodes/%s/traits/%s' % (self.node.id, 'CUSTOM_FAKE'),
+            'nodes/{}/traits/{}'.format(self.node.id, 'CUSTOM_FAKE'),
             headers=mock.ANY,
             microversion='1.37',
             retriable_status_codes=_common.RETRIABLE_STATUS_CODES,
@@ -1048,7 +1157,7 @@ class TestNodeTraits(base.TestCase):
             self.node.remove_trait(self.session, 'CUSTOM_MISSING')
         )
         self.session.delete.assert_called_once_with(
-            'nodes/%s/traits/%s' % (self.node.id, 'CUSTOM_MISSING'),
+            'nodes/{}/traits/{}'.format(self.node.id, 'CUSTOM_MISSING'),
             headers=mock.ANY,
             microversion='1.37',
             retriable_status_codes=_common.RETRIABLE_STATUS_CODES,
@@ -1058,7 +1167,7 @@ class TestNodeTraits(base.TestCase):
         traits = ['CUSTOM_FAKE', 'CUSTOM_REAL', 'CUSTOM_MISSING']
         self.node.set_traits(self.session, traits)
         self.session.put.assert_called_once_with(
-            'nodes/%s/traits' % self.node.id,
+            f'nodes/{self.node.id}/traits',
             json={'traits': ['CUSTOM_FAKE', 'CUSTOM_REAL', 'CUSTOM_MISSING']},
             headers=mock.ANY,
             microversion='1.37',
@@ -1070,7 +1179,7 @@ class TestNodeTraits(base.TestCase):
 @mock.patch.object(resource.Resource, 'patch', autospec=True)
 class TestNodePatch(base.TestCase):
     def setUp(self):
-        super(TestNodePatch, self).setUp()
+        super().setUp()
         self.node = node.Node(**FAKE)
         self.session = mock.Mock(
             spec=adapter.Adapter, default_microversion=None
@@ -1113,7 +1222,7 @@ class TestNodePatch(base.TestCase):
 @mock.patch.object(node.Node, 'fetch', autospec=True)
 class TestNodeWaitForPowerState(base.TestCase):
     def setUp(self):
-        super(TestNodeWaitForPowerState, self).setUp()
+        super().setUp()
         self.node = node.Node(**FAKE)
         self.session = mock.Mock()
 
@@ -1143,9 +1252,9 @@ class TestNodeWaitForPowerState(base.TestCase):
 @mock.patch.object(utils, 'pick_microversion', lambda session, v: v)
 @mock.patch.object(node.Node, 'fetch', lambda self, session: self)
 @mock.patch.object(exceptions, 'raise_from_response', mock.Mock())
-class TestNodePassthru(object):
+class TestNodePassthru:
     def setUp(self):
-        super(TestNodePassthru, self).setUp()
+        super().setUp()
         self.node = node.Node(**FAKE)
         self.session = node.Mock(
             spec=adapter.Adapter, default_microversion='1.37'
@@ -1155,7 +1264,7 @@ class TestNodePassthru(object):
     def test_get_passthru(self):
         self.node.call_vendor_passthru(self.session, "GET", "test_method")
         self.session.get.assert_called_once_with(
-            'nodes/%s/vendor_passthru?method=test_method' % self.node.id,
+            f'nodes/{self.node.id}/vendor_passthru?method=test_method',
             headers=mock.ANY,
             microversion='1.37',
             retriable_status_codes=_common.RETRIABLE_STATUS_CODES,
@@ -1164,7 +1273,7 @@ class TestNodePassthru(object):
     def test_post_passthru(self):
         self.node.call_vendor_passthru(self.session, "POST", "test_method")
         self.session.post.assert_called_once_with(
-            'nodes/%s/vendor_passthru?method=test_method' % self.node.id,
+            f'nodes/{self.node.id}/vendor_passthru?method=test_method',
             headers=mock.ANY,
             microversion='1.37',
             retriable_status_codes=_common.RETRIABLE_STATUS_CODES,
@@ -1173,7 +1282,7 @@ class TestNodePassthru(object):
     def test_put_passthru(self):
         self.node.call_vendor_passthru(self.session, "PUT", "test_method")
         self.session.put.assert_called_once_with(
-            'nodes/%s/vendor_passthru?method=test_method' % self.node.id,
+            f'nodes/{self.node.id}/vendor_passthru?method=test_method',
             headers=mock.ANY,
             microversion='1.37',
             retriable_status_codes=_common.RETRIABLE_STATUS_CODES,
@@ -1182,7 +1291,7 @@ class TestNodePassthru(object):
     def test_delete_passthru(self):
         self.node.call_vendor_passthru(self.session, "DELETE", "test_method")
         self.session.delete.assert_called_once_with(
-            'nodes/%s/vendor_passthru?method=test_method' % self.node.id,
+            f'nodes/{self.node.id}/vendor_passthru?method=test_method',
             headers=mock.ANY,
             microversion='1.37',
             retriable_status_codes=_common.RETRIABLE_STATUS_CODES,
@@ -1191,7 +1300,7 @@ class TestNodePassthru(object):
     def test_list_passthru(self):
         self.node.list_vendor_passthru(self.session)
         self.session.get.assert_called_once_with(
-            'nodes/%s/vendor_passthru/methods' % self.node.id,
+            f'nodes/{self.node.id}/vendor_passthru/methods',
             headers=mock.ANY,
             microversion='1.37',
             retriable_status_codes=_common.RETRIABLE_STATUS_CODES,
@@ -1212,7 +1321,7 @@ class TestNodeConsole(base.TestCase):
     def test_get_console(self):
         self.node.get_console(self.session)
         self.session.get.assert_called_once_with(
-            'nodes/%s/states/console' % self.node.id,
+            f'nodes/{self.node.id}/states/console',
             headers=mock.ANY,
             microversion=mock.ANY,
             retriable_status_codes=_common.RETRIABLE_STATUS_CODES,
@@ -1221,7 +1330,7 @@ class TestNodeConsole(base.TestCase):
     def test_set_console_mode(self):
         self.node.set_console_mode(self.session, True)
         self.session.put.assert_called_once_with(
-            'nodes/%s/states/console' % self.node.id,
+            f'nodes/{self.node.id}/states/console',
             json={'enabled': True},
             headers=mock.ANY,
             microversion=mock.ANY,
@@ -1273,8 +1382,53 @@ class TestNodeInventory(base.TestCase):
         self.assertEqual(node_inventory, res)
 
         self.session.get.assert_called_once_with(
-            'nodes/%s/inventory' % self.node.id,
+            f'nodes/{self.node.id}/inventory',
             headers=mock.ANY,
             microversion='1.81',
             retriable_status_codes=_common.RETRIABLE_STATUS_CODES,
         )
+
+
+@mock.patch.object(node.Node, 'fetch', lambda self, session: self)
+@mock.patch.object(exceptions, 'raise_from_response', mock.Mock())
+class TestNodeFirmware(base.TestCase):
+    def setUp(self):
+        super().setUp()
+        self.node = node.Node(**FAKE)
+        self.session = mock.Mock(
+            spec=adapter.Adapter,
+            default_microversion='1.86',
+        )
+
+    def test_list_firmware(self):
+        node_firmware = {
+            "firmware": [
+                {
+                    "created_at": "2016-08-18T22:28:49.653974+00:00",
+                    "updated_at": "2016-08-18T22:28:49.653974+00:00",
+                    "component": "BMC",
+                    "initial_version": "v1.0.0",
+                    "current_version": "v1.2.0",
+                    "last_version_flashed": "v1.2.0",
+                },
+                {
+                    "created_at": "2016-08-18T22:28:49.653974+00:00",
+                    "updated_at": "2016-08-18T22:28:49.653974+00:00",
+                    "component": "BIOS",
+                    "initial_version": "v1.0.0",
+                    "current_version": "v1.1.5",
+                    "last_version_flashed": "v1.1.5",
+                },
+            ]
+        }
+        self.session.get.return_value.json.return_value = node_firmware
+
+        res = self.node.list_firmware(self.session)
+        self.assertEqual(node_firmware, res)
+
+        self.session.get.assert_called_once_with(
+            f'nodes/{self.node.id}/firmware',
+            headers=mock.ANY,
+            microversion='1.86',
+            retriable_status_codes=_common.RETRIABLE_STATUS_CODES,
+        )
