@@ -88,7 +88,7 @@ class TestCase(base.TestCase):
     def setUp(self, cloud_config_fixture='clouds.yaml'):
         """Run before each test method to initialize test environment."""
 
-        super(TestCase, self).setUp()
+        super().setUp()
 
         # Sleeps are for real testing, but unit tests shouldn't need them
         realsleep = time.sleep
@@ -203,8 +203,8 @@ class TestCase(base.TestCase):
         if append:
             to_join.extend([urllib.parse.quote(i) for i in append])
         if qs_elements is not None:
-            qs = '?%s' % '&'.join(qs_elements)
-        return '%(uri)s%(qs)s' % {'uri': '/'.join(to_join), 'qs': qs}
+            qs = '?{}'.format('&'.join(qs_elements))
+        return '{uri}{qs}'.format(uri='/'.join(to_join), qs=qs)
 
     def mock_for_keystone_projects(
         self,
@@ -403,7 +403,7 @@ class TestCase(base.TestCase):
         self, type=None, name=None, description=None, enabled=True
     ):
         service_id = uuid.uuid4().hex
-        name = name or uuid.uuid4().hex
+        name = name or f'name-{uuid.uuid4().hex}'
         type = type or uuid.uuid4().hex
 
         response = {
@@ -481,7 +481,7 @@ class TestCase(base.TestCase):
                 dict(
                     method='GET',
                     uri='https://identity.example.com/',
-                    text=open(self.discovery_json, 'r').read(),
+                    text=open(self.discovery_json).read(),
                 ),
                 dict(
                     method='POST',
@@ -532,7 +532,7 @@ class TestCase(base.TestCase):
         )
 
     def get_keystone_discovery(self):
-        with open(self.discovery_json, 'r') as discovery_file:
+        with open(self.discovery_json) as discovery_file:
             return dict(
                 method='GET',
                 uri='https://identity.example.com/',
@@ -591,7 +591,7 @@ class TestCase(base.TestCase):
         return dict(
             method='GET',
             uri=block_storage_discovery_url,
-            text=open(discovery_fixture, 'r').read(),
+            text=open(discovery_fixture).read(),
         )
 
     def get_glance_discovery_mock_dict(
@@ -606,7 +606,7 @@ class TestCase(base.TestCase):
             method='GET',
             uri=image_discovery_url,
             status_code=300,
-            text=open(discovery_fixture, 'r').read(),
+            text=open(discovery_fixture).read(),
         )
 
     def get_nova_discovery_mock_dict(
@@ -620,7 +620,7 @@ class TestCase(base.TestCase):
         return dict(
             method='GET',
             uri=compute_discovery_url,
-            text=open(discovery_fixture, 'r').read(),
+            text=open(discovery_fixture).read(),
         )
 
     def get_placement_discovery_mock_dict(
@@ -632,7 +632,7 @@ class TestCase(base.TestCase):
         return dict(
             method='GET',
             uri="https://placement.example.com/",
-            text=open(discovery_fixture, 'r').read(),
+            text=open(discovery_fixture).read(),
         )
 
     def get_designate_discovery_mock_dict(self):
@@ -640,7 +640,7 @@ class TestCase(base.TestCase):
         return dict(
             method='GET',
             uri="https://dns.example.com/",
-            text=open(discovery_fixture, 'r').read(),
+            text=open(discovery_fixture).read(),
         )
 
     def get_ironic_discovery_mock_dict(self):
@@ -650,7 +650,7 @@ class TestCase(base.TestCase):
         return dict(
             method='GET',
             uri="https://baremetal.example.com/",
-            text=open(discovery_fixture, 'r').read(),
+            text=open(discovery_fixture).read(),
         )
 
     def get_senlin_discovery_mock_dict(self):
@@ -660,7 +660,7 @@ class TestCase(base.TestCase):
         return dict(
             method='GET',
             uri="https://clustering.example.com/",
-            text=open(discovery_fixture, 'r').read(),
+            text=open(discovery_fixture).read(),
         )
 
     def use_compute_discovery(
@@ -683,7 +683,7 @@ class TestCase(base.TestCase):
         return dict(
             method='GET',
             uri="https://accelerator.example.com/",
-            text=open(discovery_fixture, 'r').read(),
+            text=open(discovery_fixture).read(),
         )
 
     def get_manila_discovery_mock_dict(self):
@@ -693,7 +693,7 @@ class TestCase(base.TestCase):
         return dict(
             method='GET',
             uri="https://shared-file-system.example.com/",
-            text=open(discovery_fixture, 'r').read(),
+            text=open(discovery_fixture).read(),
         )
 
     def use_glance(
@@ -811,23 +811,19 @@ class TestCase(base.TestCase):
             # NOTE(notmorgan): make sure the delimiter is non-url-safe, in this
             # case "|" is used so that the split can be a bit easier on
             # maintainers of this code.
-            key = '{method}|{uri}|{params}'.format(
-                method=method, uri=uri, params=kw_params
-            )
+            key = f'{method}|{uri}|{kw_params}'
             validate = to_mock.pop('validate', {})
-            valid_keys = set(['json', 'headers', 'params', 'data'])
+            valid_keys = {'json', 'headers', 'params', 'data'}
             invalid_keys = set(validate.keys()) - valid_keys
             if invalid_keys:
                 raise TypeError(
-                    "Invalid values passed to validate: {keys}".format(
-                        keys=invalid_keys
-                    )
+                    f"Invalid values passed to validate: {invalid_keys}"
                 )
             headers = structures.CaseInsensitiveDict(
                 to_mock.pop('headers', {})
             )
             if 'content-type' not in headers:
-                headers[u'content-type'] = 'application/json'
+                headers['content-type'] = 'application/json'
 
             if 'exc' not in to_mock:
                 to_mock['headers'] = headers
@@ -841,11 +837,10 @@ class TestCase(base.TestCase):
                     'PROGRAMMING ERROR: key-word-params '
                     'should be part of the uri_key and cannot change, '
                     'it will affect the matcher in requests_mock. '
-                    '%(old)r != %(new)r'
-                    % {
-                        'old': self._uri_registry[key]['kw_params'],
-                        'new': kw_params,
-                    }
+                    '{old!r} != {new!r}'.format(
+                        old=self._uri_registry[key]['kw_params'],
+                        new=kw_params,
+                    )
                 )
             self._uri_registry[key]['response_list'].append(to_mock)
 
@@ -855,7 +850,7 @@ class TestCase(base.TestCase):
                 mock_method,
                 mock_uri,
                 params['response_list'],
-                **params['kw_params']
+                **params['kw_params'],
             )
 
     def assert_no_calls(self):
@@ -892,25 +887,17 @@ class TestCase(base.TestCase):
                     urllib.parse.parse_qs(history_uri_parts.query),
                 ),
                 (
-                    'REST mismatch on call %(index)d. Expected %(call)r. '
-                    'Got %(history)r). '
+                    f'REST mismatch on call {x}. '
+                    f'Expected {call["method"]} {call["url"]}. '
+                    f'Got {history.method} {history.url}. '
                     'NOTE: query string order differences wont cause mismatch'
-                    % {
-                        'index': x,
-                        'call': '{method} {url}'.format(
-                            method=call['method'], url=call['url']
-                        ),
-                        'history': '{method} {url}'.format(
-                            method=history.method, url=history.url
-                        ),
-                    }
                 ),
             )
             if 'json' in call:
                 self.assertEqual(
                     call['json'],
                     history.json(),
-                    'json content mismatch in call {index}'.format(index=x),
+                    f'json content mismatch in call {x}',
                 )
             # headers in a call isn't exhaustive - it's checking to make sure
             # a specific header or headers are there, not that they are the
@@ -920,15 +907,22 @@ class TestCase(base.TestCase):
                     self.assertEqual(
                         value,
                         history.headers[key],
-                        'header mismatch in call {index}'.format(index=x),
+                        f'header mismatch in call {x}',
                     )
         if do_count:
             self.assertEqual(
                 len(self.calls),
                 len(self.adapter.request_history),
                 "Expected:\n{}'\nGot:\n{}".format(
-                    '\n'.join([c['url'] for c in self.calls]),
-                    '\n'.join([h.url for h in self.adapter.request_history]),
+                    '\n'.join(
+                        [f'{c["method"]} {c["url"]}' for c in self.calls]
+                    ),
+                    '\n'.join(
+                        [
+                            f'{h.method} {h.url}'
+                            for h in self.adapter.request_history
+                        ]
+                    ),
                 ),
             )
 
@@ -964,7 +958,7 @@ class TestCase(base.TestCase):
 
 class IronicTestCase(TestCase):
     def setUp(self):
-        super(IronicTestCase, self).setUp()
+        super().setUp()
         self.use_ironic()
         self.uuid = str(uuid.uuid4())
         self.name = self.getUniqueString('name')
@@ -973,4 +967,4 @@ class IronicTestCase(TestCase):
         kwargs.setdefault('service_type', 'baremetal')
         kwargs.setdefault('interface', 'public')
         kwargs.setdefault('base_url_append', 'v1')
-        return super(IronicTestCase, self).get_mock_url(**kwargs)
+        return super().get_mock_url(**kwargs)
