@@ -88,7 +88,7 @@ class TestSnapshot(base.TestCase):
 
 class TestSnapshotActions(base.TestCase):
     def setUp(self):
-        super(TestSnapshotActions, self).setUp()
+        super().setUp()
         self.resp = mock.Mock()
         self.resp.body = None
         self.resp.json = mock.Mock(return_value=self.resp.body)
@@ -105,7 +105,7 @@ class TestSnapshotActions(base.TestCase):
 
         self.assertIsNone(sot.force_delete(self.sess))
 
-        url = 'snapshots/%s/action' % FAKE_ID
+        url = f'snapshots/{FAKE_ID}/action'
         body = {'os-force_delete': None}
         self.sess.post.assert_called_with(
             url, json=body, microversion=sot._max_microversion
@@ -116,7 +116,7 @@ class TestSnapshotActions(base.TestCase):
 
         self.assertIsNone(sot.reset(self.sess, 'new_status'))
 
-        url = 'snapshots/%s/action' % FAKE_ID
+        url = f'snapshots/{FAKE_ID}/action'
         body = {'os-reset_status': {'status': 'new_status'}}
         self.sess.post.assert_called_with(
             url, json=body, microversion=sot._max_microversion
@@ -127,7 +127,7 @@ class TestSnapshotActions(base.TestCase):
 
         self.assertIsNone(sot.set_status(self.sess, 'new_status'))
 
-        url = 'snapshots/%s/action' % FAKE_ID
+        url = f'snapshots/{FAKE_ID}/action'
         body = {'os-update_snapshot_status': {'status': 'new_status'}}
         self.sess.post.assert_called_with(
             url, json=body, microversion=sot._max_microversion
@@ -206,7 +206,7 @@ class TestSnapshotActions(base.TestCase):
 
         self.assertIsNone(sot.unmanage(self.sess))
 
-        url = 'snapshots/%s/action' % FAKE_ID
+        url = f'snapshots/{FAKE_ID}/action'
         body = {'os-unmanage': None}
         self.sess.post.assert_called_with(
             url, json=body, microversion=sot._max_microversion
