@@ -36,7 +36,7 @@ class TestProject(base.TestCase):
             resource = 'tenants'
         if base_url_append is None and v3:
             base_url_append = 'v3'
-        return super(TestProject, self).get_mock_url(
+        return super().get_mock_url(
             service_type=service_type,
             interface=interface,
             resource=resource,
@@ -122,15 +122,7 @@ class TestProject(base.TestCase):
                 ),
             ]
         )
-        # NOTE(notmorgan): This test (and shade) does not represent a case
-        # where the project is in the project list but a 404 is raised when
-        # the PATCH is issued. This is a bug in shade and should be fixed,
-        # shade will raise an attribute error instead of the proper
-        # project not found exception.
-        with testtools.ExpectedException(
-            exceptions.SDKException,
-            "Project %s not found." % project_data.project_id,
-        ):
+        with testtools.ExpectedException(exceptions.NotFoundException):
             self.cloud.update_project(project_data.project_id)
         self.assert_calls()
 
@@ -187,7 +179,7 @@ class TestProject(base.TestCase):
                     method='GET',
                     uri=self.get_mock_url(
                         resource=(
-                            'projects?domain_id=%s' % project_data.domain_id
+                            f'projects?domain_id={project_data.domain_id}'
                         )
                     ),
                     status_code=200,
@@ -212,7 +204,7 @@ class TestProject(base.TestCase):
                     method='GET',
                     uri=self.get_mock_url(
                         resource=(
-                            'projects?domain_id=%s' % project_data.domain_id
+                            f'projects?domain_id={project_data.domain_id}'
                         )
                     ),
                     status_code=200,
@@ -258,7 +250,7 @@ class TestProject(base.TestCase):
                     method='GET',
                     uri=self.get_mock_url(
                         resource=(
-                            'projects?domain_id=%s' % project_data.domain_id
+                            f'projects?domain_id={project_data.domain_id}'
                         )
                     ),
                     status_code=200,
