@@ -13,9 +13,12 @@
 from openstack.dns.v2 import _proxy
 from openstack.dns.v2 import floating_ip
 from openstack.dns.v2 import recordset
+from openstack.dns.v2 import service_status
+from openstack.dns.v2 import tsigkey
 from openstack.dns.v2 import zone
 from openstack.dns.v2 import zone_export
 from openstack.dns.v2 import zone_import
+from openstack.dns.v2 import zone_nameserver
 from openstack.dns.v2 import zone_share
 from openstack.dns.v2 import zone_transfer
 from openstack.tests.unit import test_proxy_base
@@ -23,7 +26,7 @@ from openstack.tests.unit import test_proxy_base
 
 class TestDnsProxy(test_proxy_base.TestProxyBase):
     def setUp(self):
-        super(TestDnsProxy, self).setUp()
+        super().setUp()
         self.proxy = _proxy.Proxy(self.session)
 
 
@@ -73,6 +76,16 @@ class TestDnsZone(TestDnsProxy):
         )
 
 
+class TestDnsZoneNameserver(TestDnsProxy):
+    def test_get_zone_nameservers(self):
+        self.verify_list(
+            self.proxy.zone_nameservers,
+            zone_nameserver.ZoneNameserver,
+            method_kwargs={'zone': 'id'},
+            expected_kwargs={'zone_id': 'id'},
+        )
+
+
 class TestDnsRecordset(TestDnsProxy):
     def test_recordset_create(self):
         self.verify_create(
@@ -311,3 +324,42 @@ class TestDnsZoneShare(TestDnsProxy):
             expected_args=[],
             expected_kwargs={'zone_id': 'zone'},
         )
+
+
+class TestDnsServiceStatus(TestDnsProxy):
+    def test_service_statuses(self):
+        self.verify_list(
+            self.proxy.service_statuses, service_status.ServiceStatus
+        )
+
+    def test_service_status_get(self):
+        self.verify_get(
+            self.proxy.get_service_status, service_status.ServiceStatus
+        )
+
+
+class TestDnsTsigKey(TestDnsProxy):
+    def test_tsigkey_create(self):
+        self.verify_create(
+            self.proxy.create_tsigkey,
+            tsigkey.TSIGKey,
+            method_kwargs={'name': 'id'},
+            expected_kwargs={'name': 'id', 'prepend_key': False},
+        )
+
+    def test_tsigkey_delete(self):
+        self.verify_delete(
+            self.proxy.delete_tsigkey,
+            tsigkey.TSIGKey,
+            True,
+            expected_kwargs={'ignore_missing': True, 'delete_shares': False},
+        )
+
+    def test_tsigkey_find(self):
+        self.verify_find(self.proxy.find_tsigkey, tsigkey.TSIGKey)
+
+    def test_tsigkey_get(self):
+        self.verify_get(self.proxy.get_tsigkey, tsigkey.TSIGKey)
+
+    def test_tesigkeys(self):
+        self.verify_list(self.proxy.tsigkeys, tsigkey.TSIGKey)
